-- ***************************************************************************
-- Script de creaci dels elements de la capa intermitja de la base de dades UOC_SGDB
-- Autor: Miguel ngel Bueno Snchez
-- TFC BBDD Relacionals
-- 2014-2015-1
-- ***************************************************************************
set serveroutput on;

-- ---------------------------------------------------------------------------
-- Procediment per gravar els registres d'execuci dels procediments emmagatzemats
-- de la interfcie pblica.
-- Parmetres:
-- NomProcediment   IN VARCHAR2   Nom del procediment pblic que s'ha cridat.
-- Tipus            IN CHAR       'E' si es registra l'entrada al procediment.
--                                'S' si es registra la sortida del procediment.
-- Parametres       IN VARCHAR2   Parmetres amb que es va cridar al procediment.
-- Resultat         IN VARCHAR2   Resultat del procediment.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procGravarLog
(
  pNomProcediment IN system.LOGPROCEDIMENTS."NOMPROCEDIMENT"%TYPE,
  pTipus IN system.LOGPROCEDIMENTS."TIPUS"%TYPE,
  pParametres IN system.LOGPROCEDIMENTS."PARAMETRES"%TYPE,
  pResultat IN system.LOGPROCEDIMENTS."RESULTAT"%TYPE
)
IS
BEGIN
  -- Gravar un registre nou a la taula de log.
  insert into LOGPROCEDIMENTS
    ("DATAHORA", "NOMPROCEDIMENT", "TIPUS", "PARAMETRES", "RESULTAT")
    values (CURRENT_TIMESTAMP, pNomProcediment, pTipus, pParametres, pResultat);
  commit;
END procGravarLog;
/

-- ---------------------------------------------------------------------------
-- Procediment per verificar si un codi de pas s o no correcte.
-- Parmetres:
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procVerificarCodiPais
(
  pCodiPais IN system.PAISOS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR
)
IS
  error_CodiPais EXCEPTION;
  quants INTEGER;
BEGIN
  -- Validaci dels parmetres.
  if (pCodiPais IS NULL or pCodiPais = '' or pCodiPais = 'UE') then
    raise error_CodiPais; 
  end if;
  
  -- Verificar si el pas indicat existeix.
  select count(*) into quants from system."PAISOS" where "CODIPAIS" = pCodiPais;
  if (quants = 0) then
    raise error_CodiPais;
  end if;

  RSP := 'OK';
  
EXCEPTION
  when error_CodiPais then
    RSP := 'ERROR:001';
  
  when others then
    RSP := 'ERROR:017';
        
END procVerificarCodiPais;
/

-- ---------------------------------------------------------------------------
-- Procediment per verificar si un ciutad existeix i est actiu (no donat de baixa.)
-- Parmetres:
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- IdCiutada        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procVerificarCiutadaAlta
(
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdCiutada IN system.CIUTADANS."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
  error_BaixaCiutada EXCEPTION;
  error_IdCiutada EXCEPTION;
  quants INTEGER;
  vActiu CHAR(1);
BEGIN
  -- Verificar els parmetres.
  if (pIdCiutada IS NULL or pIdCiutada = '') then
    raise error_IdCiutada;
  end if;

  -- Verificar si el ciutad existeix i no est donat de baixa.
  select "ACTIU" into vActiu from system."CIUTADANS" 
    where "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  if (vActiu = 'N') then
    raise error_BaixaCiutada;
  end if;
  
  RSP := 'OK';
  
EXCEPTION
  when NO_DATA_FOUND then
    RSP := 'ERROR:007';
    
  when error_IdCiutada then
    RSP := 'ERROR:002';
  
  when error_BaixaCiutada then
    RSP := 'ERROR:008';
  
  when others then
    RSP := 'ERROR:017';
        
END procVerificarCiutadaAlta;
/

-- ---------------------------------------------------------------------------
-- Procediment per verificar si un codi de malaltia s o no correcte.
-- Parmetres:
-- pCodiMalaltia    IN INTEGER    Codi de la malaltia.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procVerificarCodiMalaltia
(
  pCodiMalaltia IN system.TIPUSMALALTIES."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR
)
IS
  error_CodiMalaltia EXCEPTION;
  quants INTEGER;
BEGIN
  -- Validaci dels parmetres.
  if (pCodiMalaltia IS NULL or pCodiMalaltia < 1) then
    raise error_CodiMalaltia; 
  end if;
  
  -- Verificar si existeix el codi de la malaltia indicada.
  select count(*) into quants 
  from system."TIPUSMALALTIES" 
  where "CODIMALALTIA" = pCodiMalaltia;
  if (quants = 0) then
    raise error_CodiMalaltia;
  end if;

  RSP := 'OK';
  
EXCEPTION
  when error_CodiMalaltia then
    RSP := 'ERROR:019';
  
  when others then
    RSP := 'ERROR:017';
        
END procVerificarCodiMalaltia;
/

-- ---------------------------------------------------------------------------
-- Procediment per crear un nou ciutad a la base de dades.
-- Parmetres:
-- CodiPais         IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- IdCiutada        IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- Nom              IN VARCHAR2   Nom del ciutad.
-- Cognoms          IN VARCHAR2   Cognoms del ciutad.
-- Sexe             IN CHAR       Sexe del ciutad. Noms els valors H (home) i D (dona).
-- Email            IN VARCHAR2   Adrea de correu electrnic del ciutad. Pot ser null.
-- pDataNaixement   IN DATE       Data de naixement.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procCrearCiutada
(
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdCiutada IN system.CIUTADANS."IDCIUTADA"%TYPE,
  pNom IN system.CIUTADANS."NOM"%TYPE,
  pCognoms IN system.CIUTADANS."COGNOMS"%TYPE,
  pSexe IN system.CIUTADANS."SEXE"%TYPE,
  pEmail IN system.CIUTADANS."EMAIL"%TYPE,
  pDataNaixement IN system.CIUTADANS."DATANAIXEMENT"%TYPE,
  RSP OUT VARCHAR2
)
IS
BEGIN
  -- Gravar un registre nou a la taula de ciutadans.
  insert into system."CIUTADANS" 
    ("CODIPAIS", "IDCIUTADA", "NOM", "COGNOMS", "SEXE", "EMAIL", "DATANAIXEMENT", "ACTIU")
    values (pCodiPais, pIdCiutada, pNom, pCognoms, pSexe, pEmail, pDataNaixement, 'S');
  
  -- Operaci realitzada correctament.
  commit;
  RSP := 'OK';
    
EXCEPTION
  when DUP_VAL_ON_INDEX then
    RSP := 'ERROR:006';
    
  when others then
    RSP := 'ERROR:017';
    
END procCrearCiutada;
/

-- ---------------------------------------------------------------------------
-- Procediment per esborrar un ciutad de la base de dades.
-- Parmetres:
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procEliminarCiutada
(
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdCiutada IN system.CIUTADANS."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
BEGIN
  -- Marcar el ciutad com a no actiu.
  update system."CIUTADANS" 
    set "ACTIU" = 'N'
    where "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;
  
  -- Operaci realitzada correctament.
  commit;
  RSP := 'OK';

EXCEPTION
  when NO_DATA_FOUND then
    RSP := 'ERROR:007';
  
  when others then
    RSP := 'ERROR:017';
        
END procEliminarCiutada;
/

-- ---------------------------------------------------------------------------
-- Procediment per modificar les dades d'un ciutad da la base de dades.
-- Parmetres:
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pNom             IN VARCHAR2   Nom del ciutad.
-- pCognoms         IN VARCHAR2   Cognoms del ciutad.
-- pSexe            IN CHAR       Sexe del ciutad. Noms els valors H (home) i D (dona).
-- pEmail           IN VARCHAR2   Adrea de correu electrnic del ciutad. Pot ser null.
-- pDataNaixement   IN DATE       Data de naixement.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActualitzarCiutada
(
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdCiutada IN system.CIUTADANS."IDCIUTADA"%TYPE,
  pNom IN system.CIUTADANS."NOM"%TYPE,
  pCognoms IN system.CIUTADANS."COGNOMS"%TYPE,
  pSexe IN system.CIUTADANS."SEXE"%TYPE,
  pEmail IN system.CIUTADANS."EMAIL"%TYPE,
  pDataNaixement IN system.CIUTADANS."DATANAIXEMENT"%TYPE,
  RSP OUT VARCHAR
)
IS
BEGIN
  -- Esborrar el ciutad.
  update system."CIUTADANS" 
    set "NOM" = pNom, 
        "COGNOMS" = pCognoms, 
        "SEXE" = pSexe, 
        "EMAIL" = pEmail, 
        "DATANAIXEMENT" = pDataNaixement,
        "ACTIU" = 'S'
    where "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;
  
  -- Operaci realitzada correctament.
  commit;
  RSP := 'OK';

EXCEPTION
  when NO_DATA_FOUND then
    RSP := 'ERROR:007';
  
  when others then
    RSP := 'ERROR:017';
        
END procActualitzarCiutada;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir una dada estadstica.
-- Parmetres:
-- pAny             IN NUMBER     Any a consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION funcObtenirValorEstadistica
(
  pCodiEstadistica IN system.ESTADISTICAVALORS."CODIESTADISTICA"%TYPE,
  pAny IN system.ESTADISTICAVALORS."ANY"%TYPE,
  pCodiPais IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  RSP OUT VARCHAR
) RETURN INTEGER
IS
  vValor INTEGER;
BEGIN
  -- Llegir la dada estadstica.
  select "VALOR" into vValor
  from "ESTADISTICAVALORS"
  where "ANY" = pAny 
    and "CODIPAIS" = pCodiPais 
    and "CODIESTADISTICA" = pCodiEstadistica;

  -- Operaci correcta.  
  RSP := 'OK';
  return vValor;

EXCEPTION
  when NO_DATA_FOUND then
    RSP := 'OK';
    return 0;
    
  when others then
    RSP := 'ERROR:017';
    return 0;
        
END funcObtenirValorEstadistica;
/

-- ---------------------------------------------------------------------------
-- Procediment per obtenir la dada estadstica del ciutad que ms calories ha gastat.
-- Parmetres:
-- pAny             IN NUMBER     Any a consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas a consultar. Pot ser la Uni Europea.
-- pCodiPaisc       OUT VARCHAR2  Codi del pas al qual pertany el ciutad.
-- pIdCiutada       OUT VARCHAR2  Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procObtenirCiutadaEstadistica
(
  pCodiEstadistica IN system.ESTADISTICACALORIES."CODIESTADISTICA"%TYPE,
  pAny IN system.ESTADISTICACALORIES."ANY"%TYPE,
  pCodiPais IN system.ESTADISTICACALORIES."CODIPAISE"%TYPE,
  pCodiPaisC OUT system.ESTADISTICACALORIES."CODIPAISC"%TYPE,
  pIdCiutada OUT system.ESTADISTICACALORIES."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
BEGIN
  -- Llegir la dada estadstica.
  select "CODIPAISC", "IDCIUTADA" into pCodiPaisC, pIdCiutada
  from "ESTADISTICACALORIES"
  where "ANY" = pAny 
    and "CODIPAISE" = pCodiPais 
    and "CODIESTADISTICA" = pCodiEstadistica;

  -- Operaci correcta.  
  RSP := 'OK';

EXCEPTION
  when NO_DATA_FOUND then
    pCodiPaisC := '*';
    pIdCiutada := '*';
    RSP := 'OK';
    
  when others then
    RSP := 'ERROR:017';
        
END procObtenirCiutadaEstadistica;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar els registres d'estadstiques.
-- Parmetres:
-- pCodiEstadistica IN INTEGER    Codi de la estadistica.
-- pAny             IN NUMBER     Any a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pValor           IN INTEGER    Valor a sumar a la estadstica.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActEstadisticaPais
(
  pCodiEstadistica IN system.ESTADISTICAVALORS."CODIESTADISTICA"%TYPE,
  pAny IN system.ESTADISTICAVALORS."ANY"%TYPE,
  pCodiPais IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  pValor IN system.ESTADISTICAVALORS."VALOR"%TYPE,
  RSP OUT VARCHAR
)
IS
  vValor INTEGER;
BEGIN
  -- Noms s'actualitzar la estadstica si el valor que es passa s diferent de 0.
  if (pValor <> 0) then
    -- Actualitzar la estadstica del pas indicat.
    vValor := funcObtenirValorEstadistica(pCodiEstadistica, pAny, pCodiPais, RSP);
    
    if (vValor = 0) then
      -- No existeix registre per aquesta estadstica. Llavors cal crear-lo nou.
      insert into system."ESTADISTICAVALORS" 
        ("ANY", "CODIESTADISTICA", "CODIPAIS", "VALOR")
        values (pAny, pCodiEstadistica, pCodiPais, pValor);
    else
      -- Calcular el nou valor de la estadstica.
      vValor := vValor + pValor;

      if (vValor = 0) then
        -- Si la estadstica quedar a 0, eliminar el registre.
        delete from system."ESTADISTICAVALORS"
        where "ESTADISTICAVALORS"."ANY" = pAny
          and "ESTADISTICAVALORS"."CODIESTADISTICA" = pCodiEstadistica
          and "ESTADISTICAVALORS"."CODIPAIS" = pCodiPais;
      else
        -- Cal actualitzar el valor sumant-li el del parmetre.
        update system."ESTADISTICAVALORS"
          set "VALOR" = vValor 
          where "ESTADISTICAVALORS"."ANY" = pAny
          and "ESTADISTICAVALORS"."CODIESTADISTICA" = pCodiEstadistica
          and "ESTADISTICAVALORS"."CODIPAIS" = pCodiPais;
      end if;    
    end if;    
  end if;
    
  -- Operaci realitzada correctament.
  RSP := 'OK';

EXCEPTION
  when others then
    RSP := 'ERROR:017';
    
END procActEstadisticaPais;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar els registres d'estadstiques.
-- Parmetres:
-- pCodiEstadistica IN INTEGER    Codi de la estadistica.
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pValor           IN INTEGER    Valor a sumar a la estadstica.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActualitzarEstadistica
(
  pCodiEstadistica IN system.ESTADISTICAVALORS."CODIESTADISTICA"%TYPE,
  pData IN DATE,
  pCodiPais IN system.ESTADISTICAVALORS."CODIPAIS"%TYPE,
  pValor IN system.ESTADISTICAVALORS."VALOR"%TYPE,
  RSP OUT VARCHAR
)
IS
  vAny NUMBER(4,0);
  quants INTEGER;
  error_CodiEstadistica EXCEPTION;
BEGIN
  -- Verificar si el codi d'estadstica indicat existeix.
  select count(*) into quants
  from system."TIPUSESTADISTIQUES"
  where "CODIESTADISTICA" = pCodiEstadistica;
  if (quants = 0) then
    raise error_CodiEstadistica;
  end if;
  
  -- Obtenir l'any.
  vAny := EXTRACT(year from pData);

  -- Actualitzar la estadstica del pas.
  procActEstadisticaPais(pCodiEstadistica, vAny, pCodiPais, pValor, RSP);
  if (RSP = 'OK') then
    -- Actualitzar la estadstica de la Uni Europea.
    procActEstadisticaPais(pCodiEstadistica, vAny, 'UE', pValor, RSP);
  end if;
  
EXCEPTION
  when error_CodiEstadistica then
    RSP := 'ERROR:020';
    
  when others then
    RSP := 'ERROR:017';
    
END procActualitzarEstadistica;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar el rnking de calories dels ciutadans, que desprs
-- permetr mantenir calulada la estadstica 9.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPaisEst     IN VARCHAR2   Codi del pas al qual pertany la estadstica.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pCalories        IN INTEGER    Nmero de calores gastades.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActRankingCaloriesPais
(
  pData IN DATE,
  pCodiPaisEst IN system.RANKINGCALORIES."CODIPAISE"%TYPE,
  pCodiPais IN system.RANKINGCALORIES."CODIPAISC"%TYPE,
  pIdCiutada IN system.RANKINGCALORIES."IDCIUTADA"%TYPE,
  pCalories IN system.RANKINGCALORIES."CALORIES"%TYPE,
  RSP OUT VARCHAR
)
IS
  vAny NUMBER(4,0);
  quants INTEGER;
  vCalories INTEGER;
  vNovesCalories INTEGER;
  vActual INTEGER;
  vCodiPais VARCHAR2(2);
  vIdCiutada VARCHAR2(20);
  error_actualitzant_ranking EXCEPTION;
BEGIN
  -- PAS 1: Actualitzar els registres del rnking acumulat de calories pel ciutad.
  vAny := EXTRACT(year from pData);
  
  begin
    -- Llegir el registre suposadament existent de rnking de calories pel ciutad.
    select "CALORIES" into vCalories
    from system."RANKINGCALORIES"
    where "RANKINGCALORIES"."ANY" = vAny
      and "RANKINGCALORIES"."CODIPAISE" = pCodiPaisEst
      and "RANKINGCALORIES"."CODIPAISC" = pCodiPais
      and "RANKINGCALORIES"."IDCIUTADA" = pIdCiutada;
      
    -- Calcular el nou valor de calories acumulades pel ciutad.  
    vCalories := vCalories + pCalories;
    
    if (vCalories = 0) then
      -- Si el nou valor s 0, eliminar-lo.
      delete from system."RANKINGCALORIES"
      where "RANKINGCALORIES"."ANY" = vAny
        and "RANKINGCALORIES"."CODIPAISE" = pCodiPaisEst
        and "RANKINGCALORIES"."CODIPAISC" = pCodiPais
        and "RANKINGCALORIES"."IDCIUTADA" = pIdCiutada;
    else     
      -- Actualitzar el registre amb el no valor.
      update system."RANKINGCALORIES"
      set "CALORIES" = vCalories
      where "RANKINGCALORIES"."ANY" = vAny
        and "RANKINGCALORIES"."CODIPAISE" = pCodiPaisEst
        and "RANKINGCALORIES"."CODIPAISC" = pCodiPais
        and "RANKINGCALORIES"."IDCIUTADA" = pIdCiutada;
    end if;    
    
  exception
    when NO_DATA_FOUND then
      -- No existeix un registre previ de rnking de calories --> Crear-lo.
      insert into system."RANKINGCALORIES" 
        ("ANY", "CODIPAISE", "CODIPAISC", "IDCIUTADA", "CALORIES")
        values (vAny, pCodiPaisEst, pCodiPais, pIdCiutada, pCalories);
  
      vCalories := pCalories;
    
    when others then 
      raise error_actualitzant_ranking;
      
  end;
  
  -- PAS 2: Actualitzar el valor de la estadstica.
  
  -- Comprovar si ha d'actualitzar la estadstica 9, que ser quan el valor 
  -- acumulat del ciutad sigui superior al valor actual de la estadstica.
  vActual := funcObtenirValorEstadistica(9, vAny, pCodiPaisEst, RSP);
  if (RSP <> 'OK') then
    return;
  end if;  
  
  if (vActual < vCalories) then
    -- PAS 3: S que s'ha d'actualitzar el valor de la estadstica per qu aquest
    -- ciutad t un acumulat superior al de la estadstica.
    vCalories := vCalories - vActual;
    procActEstadisticaPais(9, vAny, pCodiPaisEst, vCalories, RSP);
    if (RSP = 'OK') then
      -- PAS 3.1: Actualitzar el id del nou ciutad que ms calories ha gastat.
      begin
        -- Provar d'actualitzar el registre si ja existeix.
        update system."ESTADISTICACALORIES"
          set "CODIPAISC" = pCodiPais, 
              "IDCIUTADA" = pIdCiutada
          where "ANY" = vAny 
            and "CODIESTADISTICA" = 9 
            and "CODIPAISE" = pCodiPaisEst;
          
        if sql%NOTFOUND then
          raise NO_DATA_FOUND;
        end if;
          
      exception
        when NO_DATA_FOUND then
          -- No existeix un registre previ d'estadstica de calories --> Crar-lo.
          insert into system."ESTADISTICACALORIES"
            ("ANY", "CODIESTADISTICA", "CODIPAISE", "CODIPAISC", "IDCIUTADA")
            values (vAny, 9, pCodiPaisEst, pCodiPais, pIdCiutada);

        when others then 
          raise error_actualitzant_ranking;

      end;
    end if;
  else
    -- PAS 4: En principi no cal actualitzar la estadstica, per podria ser
    -- que aquesta fs del mateix ciutad, i aquest hagi redut el seu acumulat
    -- i ara ja no li toqui ser el ciutad que ms calories ha gastat, per qu 
    -- ara ho sigui un altre ciutad.
      
    begin
      -- Comprovar si la estadstica existent s seva, del mateix ciutad.
      select "CODIPAISC", "IDCIUTADA" 
      into vCodiPais, vIdCiutada
      from system."ESTADISTICACALORIES"
      where "ANY" = vAny 
        and "CODIESTADISTICA" = 9 
        and "CODIPAISE" = pCodiPaisEst;

      if (vCodiPais = pCodiPais and vIdCiutada = pIdCiutada) then
        -- Doncs s que era seva. 
        
        -- Ara comprovarem si continua o no essent el ciutad que ms calories ha cremat,
        -- fent servir l'ndex IDX_RANKING_CALORIES de la taula del rnking de calories i
        -- agafant nomes el primer registre segons aquest ndex.
        select "CODIPAISC", "IDCIUTADA", "CALORIES"
        into vCodiPais, vIdCiutada, vNovesCalories
        from system."RANKINGCALORIES"
        where "RANKINGCALORIES"."ANY" = vAny
          and "RANKINGCALORIES"."CODIPAISE" = pCodiPaisEst
          and ROWNUM <= 1;
        
        if (vCodiPais = pCodiPais and vIdCiutada = pIdCiutada) then
          -- s el mateix ciutad, per sembla que ha redut el seu nombre de calories.
          -- Cal adjustar el valor de la estadstica.
          vCalories := vCalories - vActual;
          procActEstadisticaPais(9, vAny, pCodiPaisEst, vCalories, RSP);
        else
          -- Ara resulta que el ciutad amb ms calories gastades s un altre.
          -- Per tant, hem d'actualitzar la estadstica de calories amb el nou ciutad.
          update system."ESTADISTICACALORIES"
            set "CODIPAISC" = vCodiPais, 
                "IDCIUTADA" = vIdCiutada
            where "ANY" = vAny 
              and "CODIESTADISTICA" = 9 
              and "CODIPAISE" = pCodiPaisEst; 
        
          -- I finalment, adjustar el valor de la estadstica.
          vNovesCalories := vNovesCalories - vActual;
          procActEstadisticaPais(9, vAny, pCodiPaisEst, vNovesCalories, RSP);
        end if;
      end if;
      
    exception
      when NO_DATA_FOUND then
        -- No existeix un registre previ d'estadstica de calories --> Crear-lo.
        insert into system."ESTADISTICACALORIES"
          ("ANY", "CODIESTADISTICA", "CODIPAISE", "CODIPAISC", "IDCIUTADA")
          values (vAny, 9, pCodiPaisEst, pCodiPais, pIdCiutada);  
          
      when others then
        raise error_actualitzant_ranking;
        
    end;
    
  end if;
  
EXCEPTION
  when error_actualitzant_ranking then
    RSP := 'ERROR:017';
  
  when others then
    RSP := 'ERROR:017';
        
END procActRankingCaloriesPais;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar el rnking de calories dels ciutadans, que desprs
-- permetr mantenir calulada la estadstica 9.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pCalories        IN INTEGER    Nmero de calores gastades.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActualitzarRankingCalories
(
  pData IN DATE,
  pCodiPais IN system.RANKINGCALORIES."CODIPAISE"%TYPE,
  pIdCiutada IN system.RANKINGCALORIES."CODIPAISC"%TYPE,
  pCalories IN system.RANKINGCALORIES."CALORIES"%TYPE,
  RSP OUT VARCHAR
)
IS
BEGIN
  procActRankingCaloriesPais(pData, pCodiPais, pCodiPais, pIdCiutada, pCalories, RSP);
  if (RSP = 'OK') then
    procActRankingCaloriesPais(pData, 'UE', pCodiPais, pIdCiutada, pCalories, RSP);
  end if;
  
END procActualitzarRankingCalories;
/

-- ---------------------------------------------------------------------------
-- Procediment per gravar registres a la taula d's del sistema per part dels ciutadans.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procAfegirUsSistema
(
  pData IN DATE,
  pCodiPais IN system.USSISTEMA."CODIPAIS"%TYPE,
  pIdCiutada IN system.USSISTEMA."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
  vAny NUMBER(4,0);
BEGIN
  -- Gravar el ciutad a la taula d's del sistema.
  vAny := EXTRACT(year from pData);
  insert into system."USSISTEMA"
    ("ANY", "CODIPAIS", "IDCIUTADA")
    values (vAny, pCodiPais, pIdCiutada);

  -- Si va gravar el nou registre, llavors s la primera vegada que aquest ciutad
  -- aporta dades al sistema, i per tant cal actualitzar la estadstica 1.
  procActualitzarEstadistica( 1, pData, pCodiPais, 1, RSP );
  
EXCEPTION
  when DUP_VAL_ON_INDEX then
    -- La duplicitat es considera que s'ha gravat correctament, per en 
    -- aquest cas no s'ha d'actualitzar la estadstica.
    RSP := 'OK';  
    
  when others then
    RSP := 'ERROR:017';
    
END procAfegirUsSistema;
/

-- ---------------------------------------------------------------------------
-- Procediment per afegir dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pPasses          IN INTEGER    Nmero de passes realitzades.
-- pCalories        IN INTEGER    Nmero de calores gastades.
-- HoresInactivitat IN INTEGER    Nmero d'hores de poca o nula activitat.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procAfegirActivitatFisica
(
  pData IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCodiPais IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdCiutada IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  pPasses IN system.ACTIVITATFISICA."PASSES"%TYPE,
  pCalories IN system.ACTIVITATFISICA."CALORIES"%TYPE,
  pHoresInactivitat IN system.ACTIVITATFISICA."HORESINACTIVITAT"%TYPE,
  RSP OUT VARCHAR
)
IS
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';
  
  -- Gravar les dades del ciutad.
  insert into system."ACTIVITATFISICA" 
    ("DATA", "CODIPAIS", "IDCIUTADA", "PASSES", "CALORIES", "HORESINACTIVITAT")
    values (pData, pCodiPais, pIdCiutada, pPasses, pCalories, pHoresInactivitat);

  -- Gravar el ciutad a la taula d's del sistema.
  procAfegirUsSistema(pData, pCodiPais, pIdCiutada, RSP);
  
  -- Actualitzar les estadstiques.
  if (RSP = 'OK' and pCalories > 0) then
    -- Suma de calories gastades.
    procActualitzarEstadistica( 2, pData, pCodiPais, pCalories, RSP );
    if (RSP = 'OK') then
      -- Rnking de calories.
      procActualitzarRankingCalories( pData, pCodiPais, pIdCiutada, pCalories, RSP );
    end if;
  end if;  
  if (RSP = 'OK' and pPasses > 0) then
    -- Suma de passes realitzades.
    procActualitzarEstadistica( 3, pData, pCodiPais, pPasses, RSP );
  end if;
  if (RSP = 'OK' and pHoresInactivitat > 0) then
    -- Suma d'hores d'inactivitat.
    procActualitzarEstadistica( 4, pData, pCodiPais, pHoresInactivitat, RSP );
  end if;
  
  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when DUP_VAL_ON_INDEX then
    rollback;
    RSP := 'ERROR:009';  
    
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procAfegirActivitatFisica;
/

-- ---------------------------------------------------------------------------
-- Procediment per eliminar dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procEliminarActivitatFisica
(
  pData IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCodiPais IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdCiutada IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  RSP OUT VARCHAR
)
IS
  vPasses INTEGER;
  vCalories INTEGER;
  vHoresInactivitat NUMBER;
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';

  -- Llegir les dades actuals.
  select "CALORIES", "PASSES", "HORESINACTIVITAT"
  into vCalories, vPasses, vHoresInactivitat
  from system."ACTIVITATFISICA" 
  where "DATA" = pData and "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  
  -- Eliminar les dades.
  delete from system."ACTIVITATFISICA" 
    where "DATA" = pData and "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;

  -- Actualitzar les estadstiques.
  if (vCalories <> 0) then
    -- Suma de calories gastades.
    vCalories := -vCalories;
    procActualitzarEstadistica( 2, pData, pCodiPais, vCalories, RSP );
    if (RSP = 'OK') then
      -- Rnking de calories.
      procActualitzarRankingCalories( pData, pCodiPais, pIdCiutada, vCalories, RSP );
    end if;
  end if;
  if (RSP = 'OK' and vPasses <> 0) then
    -- Suma de passes realitzades.
    vPasses := -vPasses;
    procActualitzarEstadistica( 3, pData, pCodiPais, vPasses, RSP );
  end if;
  if (RSP = 'OK' and vHoresInactivitat <> 0) then
    -- Suma d'hores d'inactivitat.
    vHoresInactivitat := -vHoresInactivitat;
    procActualitzarEstadistica( 4, pData, pCodiPais, vHoresInactivitat, RSP );
  end if;
  
  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when NO_DATA_FOUND then
    rollback;
    RSP := 'ERROR:010';
    
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procEliminarActivitatFisica;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar dades de l'activitat fsica d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pPasses          IN INTEGER    Nmero de passes realitzades.
-- pCalories        IN INTEGER    Nmero de calores gastades.
-- HoresInactivitat IN INTEGER    Nmero d'hores de poca o nula activitat.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActualitzaActivitatFisica
(
  pData IN system.ACTIVITATFISICA."DATA"%TYPE,
  pCodiPais IN system.ACTIVITATFISICA."CODIPAIS"%TYPE,
  pIdCiutada IN system.ACTIVITATFISICA."IDCIUTADA"%TYPE,
  pPasses IN system.ACTIVITATFISICA."PASSES"%TYPE,
  pCalories IN system.ACTIVITATFISICA."CALORIES"%TYPE,
  pHoresInactivitat IN system.ACTIVITATFISICA."HORESINACTIVITAT"%TYPE,
  RSP OUT VARCHAR
)
IS
  vPasses INTEGER;
  vCalories INTEGER;
  vHoresInactivitat NUMBER;
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';
  
  -- Llegir les dades actuals.
  select "CALORIES", "PASSES", "HORESINACTIVITAT"
  into vCalories, vPasses, vHoresInactivitat
  from system."ACTIVITATFISICA" 
  where "DATA" = pData and "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
    
  -- Gravar les noves dades.
  update system."ACTIVITATFISICA" 
    set "PASSES" = pPasses, "CALORIES" = pCalories, "HORESINACTIVITAT" = pHoresInactivitat
    where "DATA" = pData and "CODIPAIS" = pCodiPais and "IDCIUTADA" = pIdCiutada;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;    

  -- Actualitzar les estadstiques.
  if (RSP = 'OK' and pCalories <> vCalories) then
    -- Suma de calories gastades.
    vCalories := -vCalories + pCalories;
    procActualitzarEstadistica( 2, pData, pCodiPais, vCalories, RSP );
    if (RSP = 'OK') then
      -- Rnking de calories.
      procActualitzarRankingCalories( pData, pCodiPais, pIdCiutada, vCalories, RSP );
    end if;
  end if;
  if (RSP = 'OK' and pPasses <> vPasses) then
    -- Suma de passes realitzades.
    vPasses := -vPasses + pPasses;
    procActualitzarEstadistica( 3, pData, pCodiPais, vPasses, RSP );
  end if;
  if (RSP = 'OK' and pHoresInactivitat <> vHoresInactivitat) then
    -- Suma d'hores d'inactivitat.
    vHoresInactivitat := -vHoresInactivitat + pHoresInactivitat;
    procActualitzarEstadistica( 4, pData, pCodiPais, vHoresInactivitat, RSP );
  end if;
  
  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when NO_DATA_FOUND then
    rollback;
    RSP := 'ERROR:010';
  
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procActualitzaActivitatFisica;
/

-- ---------------------------------------------------------------------------
-- Procediment per afegir dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pCodiMalaltia    IN INTEGER    Codi de la malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procAfegirDadesSalut
(
  pData IN system.ESTATSALUT."DATA"%TYPE,
  pCodiPais IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdCiutada IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pCodiMalaltia IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR
)
IS
  quants INTEGER;
  vAny NUMBER(4,0);
  vDiaInicial DATE;
  vDiaFinal DATE;
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';
  
  -- Comptar quants registres ja hi han al mateix any, del mateix ciutad i
  -- codi de malaltia, per saber quants ciutadans individuals hi han patit
  -- aquesta malaltia.
  vAny := EXTRACT(year from pData);
  vDiaInicial := TO_DATE('01-01-' || vAny, 'DD-MM-YYYY');
  vDiaFinal := TO_DATE('31-12-' || vAny, 'DD-MM-YYYY');
  select count(*) into quants
  from system."ESTATSALUT" 
  where "CODIPAIS" = pCodiPais
    and "IDCIUTADA" = pIdCiutada
    and "CODIMALALTIA" = pCodiMalaltia
    and "DATA" between vDiaInicial and vDiaFinal;
  
  -- Gravar les dades.
  insert into system."ESTATSALUT" 
    ("DATA", "CODIPAIS", "IDCIUTADA", "CODIMALALTIA")
    values (pData, pCodiPais, pIdCiutada, pCodiMalaltia);
  
  -- Gravar el ciutad a la taula d's del sistema.
  procAfegirUsSistema(pData, pCodiPais, pIdCiutada, RSP);
  
  -- Actualitzar les estadstiques.
  if (RSP = 'OK') then
    if (pCodiMalaltia = 2) then
      -- Nmero total de malalties lleus.
      procActualitzarEstadistica( 8, pData, pCodiPais, 1, RSP );
    elsif (pCodiMalaltia = 3 and quants = 0) then
      -- Ciutadans amb malalties cardaques.
      procActualitzarEstadistica( 5, pData, pCodiPais, 1, RSP );
    elsif (pCodiMalaltia = 4 and quants = 0) then
      -- Ciutadans amb malalties cancergenes.
      procActualitzarEstadistica( 6, pData, pCodiPais, 1, RSP );
    elsif (pCodiMalaltia = 5 and quants = 0) then
      -- Ciutadans amb malalties degeneratives.
      procActualitzarEstadistica( 7, pData, pCodiPais, 1, RSP );
    end if;  
  end if;
  
  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when DUP_VAL_ON_INDEX then
    rollback;
    RSP := 'ERROR:011';  
  
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procAfegirDadesSalut;
/

-- ---------------------------------------------------------------------------
-- Procediment per eliminar dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pCodiMalaltia    IN INTEGER    Codi de la malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procEliminarDadesSalut
(
  pData IN system.ESTATSALUT."DATA"%TYPE,
  pCodiPais IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdCiutada IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pCodiMalaltia IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR
)
IS
  quants INTEGER;
  vAny NUMBER(4,0);
  vDiaInicial DATE;
  vDiaFinal DATE;
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';

  -- Eliminar les dades.
  delete from system."ESTATSALUT" 
  where "DATA" = pData 
    and "CODIPAIS" = pCodiPais 
    and "IDCIUTADA" = pIdCiutada 
    and "CODIMALALTIA" = pCodiMalaltia;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;

  -- Comptar quants registres ja hi han al mateix any, del mateix ciutad i
  -- codi de malaltia, per saber quants ciutadans individuals hi han patit
  -- aquesta malaltia.
  vAny := EXTRACT(year from pData);
  vDiaInicial := TO_DATE('01-01-' || vAny, 'DD-MM-YYYY');
  vDiaFinal := TO_DATE('31-12-' || vAny, 'DD-MM-YYYY');
  select count(*) into quants
  from system."ESTATSALUT" 
  where "CODIPAIS" = pCodiPais
    and "IDCIUTADA" = pIdCiutada
    and "CODIMALALTIA" = pCodiMalaltia
    and "DATA" between vDiaInicial and vDiaFinal;

  if (pCodiMalaltia = 2) then
    -- Nmero total de malalties lleus.
    procActualitzarEstadistica( 8, pData, pCodiPais, -1, RSP );
  end if;
  
  if (RSP = 'OK') then
    if (pCodiMalaltia = 3 and quants = 0) then
      -- Ciutadans amb malalties cardaques.
      procActualitzarEstadistica( 5, pData, pCodiPais, -1, RSP );
    end if;
  end if;
  
  if (RSP = 'OK') then
    if (pCodiMalaltia = 4 and quants = 0) then
      -- Ciutadans amb malalties cancergenes.
      procActualitzarEstadistica( 6, pData, pCodiPais, -1, RSP );
    end if;
  end if;
  
  if (RSP = 'OK') then
    if (pCodiMalaltia = 5 and quants = 0) then
      -- Ciutadans amb malalties degeneratives.
      procActualitzarEstadistica( 7, pData, pCodiPais, -1, RSP );
    end if;  
  end if;
  
  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when NO_DATA_FOUND then
    rollback;
    RSP := 'ERROR:012';
    
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procEliminarDadesSalut;
/

-- ---------------------------------------------------------------------------
-- Procediment per actualitzar dades de l'estat de salut d'un ciutad.
-- Parmetres:
-- pData            IN DATE       Data a que fan referncia les dades.
-- pCodiPais        IN VARCHAR2   Codi del pas al qual pertany el ciutad.
-- pIdCiutada       IN VARCHAR2   Nmero identificador del ciutad en el seu pas.
-- pCodiMalaltia    IN INTEGER    Codi de la malatia que ha sofert.
-- pNovaMalaltia    IN INTEGER    Codi de la nova malatia que ha sofert.
-- RSP              OUT VARCHAR2  Resultat de la operaci.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE procActualitzarDadesSalut
(
  pData IN system.ESTATSALUT."DATA"%TYPE,
  pCodiPais IN system.ESTATSALUT."CODIPAIS"%TYPE,
  pIdCiutada IN system.ESTATSALUT."IDCIUTADA"%TYPE,
  pCodiMalaltia IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  pNovaMalaltia IN system.ESTATSALUT."CODIMALALTIA"%TYPE,
  RSP OUT VARCHAR
)
IS
  vAny NUMBER(4,0);
  quantsAnt INTEGER;
  quantsNou INTEGER;
  vDiaInicial DATE;
  vDiaFinal DATE;
BEGIN
  set transaction name 'transaccio';
  RSP := 'OK';

  -- Actualitzar les dades.
  update system."ESTATSALUT" 
  set "CODIMALALTIA" = pNovaMalaltia
  where "DATA" = pData 
    and "CODIPAIS" = pCodiPais 
    and "IDCIUTADA" = pIdCiutada 
    and "CODIMALALTIA" = pCodiMalaltia;
  if sql%NOTFOUND then
    raise NO_DATA_FOUND;
  end if;

  -- Comptar quants registres ja hi han al mateix any, del mateix ciutad i
  -- codi de malaltia, per saber quants ciutadans individuals hi han patit
  -- aquesta malaltia.
  vAny := EXTRACT(year from pData);
  vDiaInicial := TO_DATE('01-01-' || vAny, 'DD-MM-YYYY');
  vDiaFinal := TO_DATE('31-12-' || vAny, 'DD-MM-YYYY');
  select count(*) into quantsAnt
  from system."ESTATSALUT" 
  where "CODIPAIS" = pCodiPais
    and "IDCIUTADA" = pIdCiutada
    and "CODIMALALTIA" = pCodiMalaltia
    and "DATA" between vDiaInicial and vDiaFinal;

  vAny := EXTRACT(year from pData);
  vDiaInicial := TO_DATE('01-01-' || vAny, 'DD-MM-YYYY');
  vDiaFinal := TO_DATE('31-12-' || vAny, 'DD-MM-YYYY');
  select count(*) into quantsNou
  from system."ESTATSALUT" 
  where "CODIPAIS" = pCodiPais
    and "IDCIUTADA" = pIdCiutada
    and "CODIMALALTIA" = pCodiMalaltia
    and "DATA" between vDiaInicial and vDiaFinal;

  -- Nmero total de malalties lleus.
  if (pCodiMalaltia = 2) then
    procActualitzarEstadistica( 8, pData, pCodiPais, -1, RSP );
  end if;
  if (pNovaMalaltia = 2) then
    procActualitzarEstadistica( 8, pData, pCodiPais, 1, RSP );
  end if;
  
  -- Ciutadans amb malalties cardaques.
  if (RSP = 'OK') then
    if (pCodiMalaltia = 3 and quantsAnt = 0) then
      procActualitzarEstadistica( 5, pData, pCodiPais, -1, RSP );
    end if;
  end if;
  if (RSP = 'OK') then
    if (pNovaMalaltia = 3 and quantsNou = 0) then
      procActualitzarEstadistica( 5, pData, pCodiPais, 1, RSP );
    end if;
  end if;
  
  -- Ciutadans amb malalties cancergenes.
  if (RSP = 'OK') then
    if (pCodiMalaltia = 4 and quantsAnt = 0) then
      procActualitzarEstadistica( 6, pData, pCodiPais, -1, RSP );
    end if;
  end if;
  if (RSP = 'OK') then
    if (pNovaMalaltia = 4 and quantsNou = 0) then
      procActualitzarEstadistica( 6, pData, pCodiPais, 1, RSP );
    end if;
  end if;
  
  -- Ciutadans amb malalties degeneratives.
  if (RSP = 'OK') then
    if (pCodiMalaltia = 5 and quantsAnt = 0) then
      procActualitzarEstadistica( 7, pData, pCodiPais, -1, RSP );
    end if;  
  end if;
  if (RSP = 'OK') then
    if (pNovaMalaltia = 5 and quantsNou = 0) then
      procActualitzarEstadistica( 7, pData, pCodiPais, 1, RSP );
    end if;  
  end if;

  if (RSP = 'OK') then
    commit;
  else
    rollback;
  end if;  
  
EXCEPTION
  when NO_DATA_FOUND then
    rollback;
    RSP := 'ERROR:012';
  
  when DUP_VAL_ON_INDEX then
    rollback;
    RSP := 'ERROR:011';
    
  when others then
    rollback;
    RSP := 'ERROR:017';
        
END procActualitzarDadesSalut;
/

-- ---------------------------------------------------------------------------
-- Vista per obtenir la llista de pasos de la Uni Europea.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE VIEW vwLlistatPaisos 
AS
  select "CODIPAIS" as "COUNTRYCODE",
         "NOM" as "NAME"
  from system."PAISOS"
  where "CODIPAIS" <> 'UE';
/

-- ---------------------------------------------------------------------------
-- Vista per obtenir la llista de les malalties registrades.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE VIEW vwLlistatMalalties
AS
  select "CODIMALALTIA" as "DISEASECODE",
         "DESCRIPCIO" as "DESCRIPTION"
  from system."TIPUSMALALTIES";
/

-- ---------------------------------------------------------------------------
-- Funci per obtenir la llista dels ciutadans d'un pas i edat determinats,
-- que no estiguin donats de baixa.
-- pCodiPais        IN VARCHAR2   Codi del pas.
-- pEdat            IN NUMBER     Edat dels ciutadans que es busquen.
-- Valor de retorn:
-- Taula            typeCiutadaTable  Taula dels ciutadans.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeCiutada 
  IS OBJECT
  (
    CodiPais CHAR(2),
    IdCiutada VARCHAR2(20),
    Nom VARCHAR2(30),
    Cognoms VARCHAR2(40)
  );
/

CREATE OR REPLACE TYPE typeCiutadaTable
  IS TABLE OF typeCiutada;
/

CREATE OR REPLACE FUNCTION funcLlistatCiutadansEdat
(
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pEdat IN NUMBER
)
RETURN typeCiutadaTable
IS
  Llistat typeCiutadaTable;
  indice NUMBER;
BEGIN
  -- Crear la collecci.
  Llistat := typeCiutadaTable();
  
  -- Obtenir la llista dels ciutadans del pas i edat indicats, que no estiguin
  -- donats de baixa.
  for ciutada in 
  (
    select "IDCIUTADA", "NOM", "COGNOMS"
      from system."CIUTADANS"
      where "CODIPAIS" = pCodiPais
        and "ACTIU" = 'S'
        and "DATANAIXEMENT" between 
          TO_DATE('01-01-' || (EXTRACT(year from current_date) - pEdat), 'DD-MM-YYYY')
          and
          TO_DATE('31-12-' || (EXTRACT(year from current_date) - pEdat), 'DD-MM-YYYY')
  )
  loop
    -- Afegir el registre al llistat.    
    Llistat.extend;
    indice := Llistat.count;
    Llistat(indice) := typeCiutada( pCodiPais, ciutada."IDCIUTADA", ciutada."NOM", ciutada."COGNOMS" );
  end loop;
   
  -- I retornar la llista.
  return Llistat;    

EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('funcLlistatCiutadansEdat: Error ' || SQLCODE || ' - ' || SQLERRM);
    return Llistat;
        
END funcLlistatCiutadansEdat;
/

-- ---------------------------------------------------------------------------
-- Funci que retorna el nmero de dies de l'any que es passi com a parmetre.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- Valor de retorn:
-- numDies          NUMBER        Nmero de dies de l'any.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION funcDiesAny
(
  pAny IN NUMBER
) 
RETURN NUMBER
IS
  numDies NUMBER;
BEGIN
  -- Calcular el nmero de des del l'any.
  numDies := 365;
  if (MOD(pAny, 4) = 0 or MOD(pAny, 100) = 0) then
    if (MOD(pAny, 400) <> 0) then
      numDies := 366;
    end if;
  end if;
    
  return numDies;
END funcDiesAny;
/

-- ---------------------------------------------------------------------------
-- Funci que retorna la llista dels ciutadans d'un pas d'una determinada
-- edat. Per a cada ciutad mostrar les seves dades d'estat de salut.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas.
-- pEdat            IN NUMBER     Edat dels ciutadans que es busquen.
-- pLlindarCalories IN INTEGER    Llindar de calories.
-- Valor de retorn:
-- Taula            typeLlistatSalutTable    Taula dels ciutadans.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeLlistatSalutItem 
  IS OBJECT
  (
    IdCiutada VARCHAR2(20),
    Nom VARCHAR(30),
    Cognoms VARCHAR(40),
    VisitesMetge INTEGER,
    MitjaCalories INTEGER,
    AnyCardiaca NUMBER null,
    AnyCancer NUMBER null,
    AnyDegenerativa NUMBER null,
    MalaltiesLleus INTEGER
  );
/

CREATE OR REPLACE TYPE typeLlistatSalutTable
  IS TABLE OF typeLlistatSalutItem;
/

CREATE OR REPLACE FUNCTION funcLlistatSalutCiutadans
(
  pAny IN NUMBER,
  pCodiPais IN system.PAISOS."CODIPAIS"%TYPE,
  pEdat IN NUMBER,
  pLlindarCalories IN INTEGER
) 
RETURN typeLlistatSalutTable
IS
  Llistat typeLlistatSalutTable;
  item typeLlistatSalutItem;
  indice NUMBER;
  dataInicial DATE;
  dataFinal DATE;
  sumaCalories INTEGER;
  numDiesAny INTEGER;
BEGIN
  -- Calcular les dates lmit de l'any demanat.
  dataInicial := TO_DATE('01-01-' || pAny, 'DD-MM-YYYY');
  dataFinal := TO_DATE('31-12-' || pAny, 'DD-MM-YYYY');

  -- Crear la collecci.
  Llistat := typeLlistatSalutTable();
  
  -- Obtenir la llista dels ciutadans del pas i edat indicats.
  for ciutada in 
  (
    select "IDCIUTADA", "NOM", "COGNOMS"
    from TABLE(funcLlistatCiutadansEdat(pCodiPais, pEdat))
  )
  loop
    -- Per cada ciutad, inicialitzar els valors.
    item := typeLlistatSalutItem(ciutada."IDCIUTADA", ciutada."NOM", ciutada."COGNOMS", 0, 0, null, null, null, 0);
    
    -- Per cada ciutad, llegir les seves malalties i acumular els valors.
    
    -- Nmero de visites al metge en aquell any.
    select count(*) 
    into item."VISITESMETGE"
    from system."ESTATSALUT"
      where "CODIPAIS" = pCodiPais
        and "IDCIUTADA" = ciutada."IDCIUTADA"    
        and "CODIMALALTIA" = 1
        and "DATA" between dataInicial and dataFinal;

    -- Nmero de malalties lleus en aquell any.
    select count(*) 
    into item."MALALTIESLLEUS"
    from system."ESTATSALUT"
      where "CODIPAIS" = pCodiPais
        and "IDCIUTADA" = ciutada."IDCIUTADA"    
        and "CODIMALALTIA" = 2
        and "DATA" between dataInicial and dataFinal;
    
    -- Any en que li van diagnosticar malaltia cardaca per primer cop.
    BEGIN
      select EXTRACT(year from "DATA")
      into item."ANYCANCER"
      from system."ESTATSALUT"
        where "CODIPAIS" = pCodiPais
          and "IDCIUTADA" = ciutada."IDCIUTADA"    
          and "CODIMALALTIA" = 3
          and ROWNUM <= 1
      order by "DATA" asc;
      
    EXCEPTION
      when NO_DATA_FOUND then
        null;
        
    END;

    -- Any en que li van diagnosticar malaltia cancergena per primer cop.
    BEGIN
      select EXTRACT(year from "DATA")
      into item."ANYCARDIACA"
      from system."ESTATSALUT"
        where "CODIPAIS" = pCodiPais
          and "IDCIUTADA" = ciutada."IDCIUTADA"    
          and "CODIMALALTIA" = 4
          and ROWNUM <= 1
      order by "DATA" asc;
      
    EXCEPTION
      when NO_DATA_FOUND then
        null;
        
    END;
    
    -- Any en que li van diagnosticar malaltia degenerativa per primer cop.
    BEGIN
      select EXTRACT(year from "DATA")
      into item."ANYDEGENERATIVA"
      from system."ESTATSALUT"
        where "CODIPAIS" = pCodiPais
          and "IDCIUTADA" = ciutada."IDCIUTADA"    
          and "CODIMALALTIA" = 5
          and ROWNUM <= 1
      order by "DATA" asc;
      
    EXCEPTION
      when NO_DATA_FOUND then
        null;
        
    END;
    
    -- Mitjanda diria de calories, segons el nmero de dies que ha aportat dades.
    BEGIN
      select count(*), sum("CALORIES")
      into numDiesAny, sumaCalories
      from system."ACTIVITATFISICA" 
      where "CODIPAIS" = pCodiPais
          and "IDCIUTADA" = ciutada."IDCIUTADA"    
          and "DATA" between dataInicial and dataFinal;
      
      -- Mitjanda diria de calories, segons el nmero de dies que ha aportat dades.
      if (numDiesAny > 0) then
        sumaCalories := sumaCalories / numDiesAny;
      else
        sumaCalories := 0;
      end if;
      
      -- Tenir en compte el llindar de calories mxim.
      if (sumaCalories < pLlindarCalories) then
        item."MITJACALORIES" := sumaCalories;
      end if;
      
    EXCEPTION
      when NO_DATA_FOUND then
        null;
        
    END;
        
    -- Afegir el registre al llistat.  
    if ( item."VISITESMETGE" <> 0
      or item."MALALTIESLLEUS" <> 0
      or item."MITJACALORIES" <> 0 
      or (item."ANYCANCER" IS NOT NULL and item."ANYCANCER" <> 0)
      or (item."ANYCARDIACA" IS NOT NULL and item."ANYCARDIACA" <> 0)
      or (item."ANYDEGENERATIVA" IS NOT NULL and item."ANYDEGENERATIVA" <> 0) ) then
        Llistat.extend;
        indice := Llistat.count;
        Llistat(indice) := item;
    end if;
    
  end loop;

  return Llistat;
   
EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('funcLlistatSalutCiutadans: Error ' || SQLCODE || ' - ' || SQLERRM);
    return Llistat;
        
END funcLlistatSalutCiutadans;
/

-- ---------------------------------------------------------------------------
-- Funci que retorna la llista dels valors totals mensuals de l'activitat fsica 
-- realitzada per un ciutad concret en un any concret.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- pCodiPais        IN VARCHAR2   Codi del pas del ciutad.
-- pIdNumber        IN VARCHAR2   Nmero identificador del ciutad.
-- Valor de retorn:
-- Taula            typeLlistatActFisTable    Taula dels valors totals mensuals.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeLlistatActFisItem 
  IS OBJECT
  (
    NumMes NUMBER,
    NomMes VARCHAR2(20),
    Passes INTEGER,
    MitjaCalories INTEGER,
    HoresInactivitat INTEGER
  );
/

CREATE OR REPLACE TYPE typeLlistatActFisTable
  IS TABLE OF typeLlistatActFisItem;
/

CREATE OR REPLACE FUNCTION funcActFisicaMensualCiutada
(
  pAny IN NUMBER,
  pCodiPais IN system.CIUTADANS."CODIPAIS"%TYPE,
  pIdCiutada IN system.CIUTADANS."IDCIUTADA"%TYPE
) 
RETURN typeLlistatActFisTable
IS
  Llistat typeLlistatActFisTable;
  item typeLlistatActFisItem;
  indice NUMBER;
  dataInicial DATE;
  dataFinal DATE;
  sumaPasses INTEGER;
  sumaCalories INTEGER;
  sumaHores INTEGER;
  numDiesAny INTEGER;
  numDiesMes NUMBER;
BEGIN
  -- Crear la collecci.
  Llistat := typeLlistatActFisTable();

  -- Calcular el nmero de des del l'any.
  numDiesAny := funcDiesAny(pAny);
  
  -- Obtenir la llista dels messos de l'any.
  for mes in 
  (
    select "NUMERO", "NOM", "NUMDIES"
    from system."MESOS"
    order by "NUMERO" asc
  )
  loop
    -- Per cada mes, inicialitzar els valors.
    item := typeLlistatActFisItem(mes."NUMERO", mes."NOM", 0, 0, 0);
    
    -- Calcular les dates lmit de cada mes.
    dataInicial := TO_DATE('01-' || mes."NUMERO" || '-' || pAny, 'DD-MM-YYYY');
    numDiesMes := mes."NUMDIES";
    if (mes."NUMERO" = 2 and numDiesAny = 366) then
      numDiesMes := numDiesMes + 1;
    end if;  
    dataFinal := TO_DATE(numDiesMes || '-' || mes."NUMERO" || '-' || pAny, 'DD-MM-YYYY');
    
    -- Llegir les dades del mes.
    select count(*), sum("PASSES"), sum("CALORIES"), sum("HORESINACTIVITAT")
    into numDiesMes, sumaPasses, sumaCalories, sumaHores
    from system."ACTIVITATFISICA"
    where "CODIPAIS" = pCodiPais
      and "IDCIUTADA" = pIdCiutada
      and "DATA" between dataInicial and dataFinal;

    -- Emmagatzemar els totals.
    item."PASSES" := sumaPasses;
    item."HORESINACTIVITAT" := sumaHores;
    
    -- Mitjanda diria de calories, segons el nmero de dies del mes amb dades.
    if (numDiesMes > 0) then
      item."MITJACALORIES" := sumaCalories / numDiesMes;
    end if;
    
    -- Afegir el registre al llistat.
    Llistat.extend;
    indice := Llistat.count;
    Llistat(indice) := item;
      
  end loop;

  return Llistat;
   
EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('funcActFisicaMensualCiutada: Error ' || SQLCODE || ' - ' || SQLERRM);
    return Llistat;
        
END funcActFisicaMensualCiutada;
/

-- ---------------------------------------------------------------------------
-- Funci que retorna la llista dels valors totals anuals de l'activitat fsica 
-- realitzada per tots els ciutadans de cada pas que tinguin una determinada edat
-- en un any concret.
-- Parmetres:
-- pAny             IN NUMBER     Any de les dades que es volen consultar.
-- pEdat            IN NUMBER     Edat dels ciutadans que es busquen.
-- Valor de retorn:
-- Taula            typeLlistatActFisPaisTable    Taula dels valors totals del pas.
-- ---------------------------------------------------------------------------
CREATE OR REPLACE TYPE typeLlistatActFisPaisItem 
  IS OBJECT
  (
    CodiPais CHAR(2),
    NomPais VARCHAR2(40),
    MitjaPasses INTEGER,
    MitjaCalories INTEGER,
    MitjaHoresInactivitat INTEGER
  );
/

CREATE OR REPLACE TYPE typeLlistatActFisPaisTable
  IS TABLE OF typeLlistatActFisPaisItem;
/

CREATE OR REPLACE FUNCTION funcActFisicaMitjaPaisos
(
  pAny IN NUMBER,
  pEdat IN NUMBER
) 
RETURN typeLlistatActFisPaisTable
IS
  Llistat typeLlistatActFisPaisTable;
  item typeLlistatActFisPaisItem;
  indice NUMBER;
  sumaPasses INTEGER;
  sumaCalories INTEGER;
  sumaHores INTEGER;
  numDiesPasses INTEGER;
  numDiesCalories INTEGER;
  numDiesHores INTEGER;
  dataInicial DATE;
  dataFinal DATE;
  dataFinalEdat DATE;
  diaIniciAny DATE;
  diaFinalAny DATE;
BEGIN
  -- Crear la collecci.
  Llistat := typeLlistatActFisPaisTable();
  
  dataInicial := TO_DATE('01-01-' || TO_CHAR(EXTRACT(year from current_date) - pEdat), 'DD-MM-YYYY');
  dataFinal := TO_DATE('31-12-' || TO_CHAR(EXTRACT(year from current_date) - pEdat), 'DD-MM-YYYY');
  
  dataFinalEdat := TO_DATE('31-12-' || TO_CHAR(EXTRACT(year from current_date)), 'DD-MM-YYYY');
  
  diaIniciAny := TO_DATE('01-01-' || TO_CHAR(pAny), 'DD-MM-YYYY');
  diaFinalAny := TO_DATE('31-12-' || TO_CHAR(pAny), 'DD-MM-YYYY');

  -- Obtenir la llista dels pasos de la Uni Europea.
  for pais in 
  (
    select "COUNTRYCODE", "NAME"
    from system."VWLLISTATPAISOS"
    order by "COUNTRYCODE" asc
  )
  loop
    -- Per cada pas, inicialitzar els valors acumulats.
    item := typeLlistatActFisPaisItem(pais."COUNTRYCODE", pais."NAME", 0, 0, 0);
    
    -- Inicialitzar els comptadors per aquest pas.
    sumaPasses := 0;
    sumaCalories := 0;
    sumaHores := 0;
    numDiesPasses := 0;
    numDiesCalories := 0;
    numDiesHores := 0;
    
    -- Llegir tots els registres de l'activitat fsica de l'any demanat
    -- dels ciutadans del pas en curs que tinguin edat inferior o igual a la indicada
    -- i que no estigin donats de baixa.
    for activitat in
    (
      select "ACTIVITATFISICA"."CODIPAIS",
             "ACTIVITATFISICA"."IDCIUTADA", 
             "ACTIVITATFISICA"."PASSES", 
             "ACTIVITATFISICA"."CALORIES", 
             "ACTIVITATFISICA"."HORESINACTIVITAT",
             "CIUTADANS"."DATANAIXEMENT"
      from system."ACTIVITATFISICA"
      join system."CIUTADANS" 
        on "CIUTADANS"."CODIPAIS" = "ACTIVITATFISICA"."CODIPAIS"
           and "CIUTADANS"."IDCIUTADA" = "ACTIVITATFISICA"."IDCIUTADA" 
      where "ACTIVITATFISICA"."CODIPAIS" = pais."COUNTRYCODE"
        and "CIUTADANS"."ACTIU" = 'S'
        and "CIUTADANS"."DATANAIXEMENT" between dataInicial and dataFinalEdat
        and "ACTIVITATFISICA"."DATA" between diaIniciAny and diaFinalAny
    )
    loop
      -- Si l'edat del ciutad s exactament la demanada, acumular passes, calories i hores, 
      -- i si l'edat del ciutad es inferior a la demanada, acumular noms passes.
      if (activitat."DATANAIXEMENT" between dataInicial and dataFinal) then
      
        if (activitat."CALORIES" > 0) then
          sumaCalories := sumaCalories + activitat."CALORIES";
          numDiesCalories := numDiesCalories + 1;
        end if;
        
        if (activitat."HORESINACTIVITAT" > 0) then
          sumaHores := sumaHores + activitat."HORESINACTIVITAT";
          numDiesHores := numDiesHores + 1;
        end if;
        
      end if;
      
      if (activitat."PASSES" > 0) then
        sumaPasses := sumaPasses + activitat."PASSES";
        numDiesPasses := numDiesPasses + 1;
      end if;
        
    end loop;
    
    -- Calcular i emmagatzemar les mitjes.
    if (numDiesPasses > 0) then
      item."MITJAPASSES" := sumaPasses / numDiesPasses;
    end if;
    if (numDiesCalories > 0) then
      item."MITJACALORIES" := sumaCalories / numDiesCalories;
    end if;
    if (numDiesHores > 0) then
      item."MITJAHORESINACTIVITAT" := sumaHores / numDiesHores;
    end if;
    
    -- Afegir el registre al llistat.    
    Llistat.extend;
    indice := Llistat.count;
    Llistat(indice) := item;
    
  end loop;

  return Llistat;
   
EXCEPTION
  when others then
    DBMS_OUTPUT.PUT_LINE('funcActFisicaMitjaPaisos: Error ' || SQLCODE || ' - ' || SQLERRM);
    return Llistat;
        
END funcActFisicaMitjaPaisos;
/

-- ***************************************************************************
-- Final del script.
-- ***************************************************************************

