-- ***************************************************************************
-- Script de creaci de la base de dades UOC_SGDB
-- Autor: Miguel ngel Bueno Snchez
-- TFC BBDD Relacionals
-- 2014-2015-1
-- ***************************************************************************

-- ---------------------------------------------------------------------------
-- Crear la taula de mesos de l'any.
-- ---------------------------------------------------------------------------
CREATE TABLE "MESOS" 
(
  "NUMERO" NUMBER(2,0) NOT NULL, 
  "NOM" VARCHAR2(40 CHAR) NOT NULL, 
  "NUMDIES" NUMBER(2,0) NOT NULL,
  CONSTRAINT "PK_MESOS" PRIMARY KEY ("NUMERO")
);

-- Carregar les dades dels messos a la taula Messos.
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (1, 'Gener', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (2, 'Febrer', 28);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (3, 'Mar', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (4, 'Abril', 30);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (5, 'Maig', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (6, 'Juny', 30);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (7, 'Juliol', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (8, 'Agost', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (9, 'Setembre', 30);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (10, 'Octubre', 31);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (11, 'Novembre', 30);
INSERT INTO "MESOS" ("NUMERO", "NOM", "NUMDIES") VALUES (12, 'Dessembre', 31);

-- ---------------------------------------------------------------------------
-- Crear la taula de pasos.
-- ---------------------------------------------------------------------------
CREATE TABLE "PAISOS" 
(
  "CODIPAIS" CHAR(2 CHAR) NOT NULL, 
  "NOM" VARCHAR2(40 CHAR) NOT NULL, 
  CONSTRAINT "PK_PAISOS" PRIMARY KEY ("CODIPAIS")
);

-- Carregar les dades dels pasos a la taula Pasos.
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('AT', 'Austria');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('BE', 'Blgica');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('BG', 'Bulgria');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('CY', 'Xipre');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('CZ', 'Txquia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('DE', 'Alemanya');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('DK', 'Dinamarca');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('EE', 'Estnia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('EL', 'Grcia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('ES', 'Espanya');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('FI', 'Finlndia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('FR', 'Frana');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('GB', 'Regne Unit');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('HR', 'Crocia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('HU', 'Hongria');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('IE', 'Irlanda');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('IT', 'Itlia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('LT', 'Litunia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('LU', 'Luxemburg');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('LV', 'Letnia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('MT', 'Malta');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('NL', 'Pasos Baixos');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('PL', 'Polnia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('PT', 'Portugal');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('RO', 'Romania');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('SE', 'Sucia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('SI', 'Eslovnia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('SK', 'Eslovquia');
INSERT INTO "PAISOS" ("CODIPAIS", "NOM") VALUES ('UE', 'Uni Europea');

-- ---------------------------------------------------------------------------
-- Crear la taula de ciutadans.
-- ---------------------------------------------------------------------------
CREATE TABLE "CIUTADANS" 
(
  "CODIPAIS" CHAR(2) NOT NULL, 
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  "NOM" VARCHAR2(30 CHAR) NOT NULL, 
  "COGNOMS" VARCHAR2(40 CHAR) NOT NULL, 
  "SEXE" CHAR(1 CHAR) NOT NULL, 
  "EMAIL" VARCHAR2(100 CHAR), 
  "ACTIU" CHAR(1 CHAR) NOT NULL, 
  "DATANAIXEMENT" DATE NOT NULL,
  CONSTRAINT "PK_CIUTADANS" PRIMARY KEY ("CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_CIUTADANS_PAISOS" 
    FOREIGN KEY ("CODIPAIS")
    REFERENCES "PAISOS" ("CODIPAIS"),
  CONSTRAINT "CK_PAISOS_SEXE" CHECK ("SEXE" = 'H' OR "SEXE" = 'D'),
  CONSTRAINT "CK_PAISOS_ACTIU" CHECK ("ACTIU" = 'S' OR "ACTIU" = 'N')
);

-- ---------------------------------------------------------------------------
-- Crear la taula de registres d'activitat fsica.
-- ---------------------------------------------------------------------------
CREATE TABLE "ACTIVITATFISICA" 
(
  "DATA" DATE NOT NULL,
  "CODIPAIS" CHAR(2 CHAR) NOT NULL, 
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  "PASSES" INTEGER NOT NULL,
  "CALORIES" INTEGER NOT NULL,
  "HORESINACTIVITAT" NUMBER(2,0) NOT NULL,
  CONSTRAINT "PK_ACTIVITATFISICA" 
    PRIMARY KEY ("DATA", "CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_ACTIVITATFISICA_CIUTADANS" 
    FOREIGN KEY ("CODIPAIS","IDCIUTADA")
    REFERENCES "CIUTADANS" ("CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "CK_PASSES" CHECK ("PASSES" >= 0),
  CONSTRAINT "CK_CALORIES" CHECK ("CALORIES" >= 0),
  CONSTRAINT "CK_HORESINACTIVITAT" 
    CHECK ("HORESINACTIVITAT" >= 0 AND "HORESINACTIVITAT" <= 24)
);

-- ---------------------------------------------------------------------------
-- Crear la taula de tipus de malalties.
-- ---------------------------------------------------------------------------
CREATE TABLE "TIPUSMALALTIES"
(
  "CODIMALALTIA" INTEGER NOT NULL PRIMARY KEY,
  "DESCRIPCIO" VARCHAR2(100 CHAR) NOT NULL 
);

-- Carregar les dades dels pasos a la taula TipusMalalties.
INSERT INTO "TIPUSMALALTIES" ("CODIMALALTIA", "DESCRIPCIO") 
  VALUES (1, 'Visita al metge');
INSERT INTO "TIPUSMALALTIES" ("CODIMALALTIA", "DESCRIPCIO") 
  VALUES (2, 'Malaltia lleu');
INSERT INTO "TIPUSMALALTIES" ("CODIMALALTIA", "DESCRIPCIO") 
  VALUES (3, 'Malaltia cardaca');
INSERT INTO "TIPUSMALALTIES" ("CODIMALALTIA", "DESCRIPCIO") 
  VALUES (4, 'Malaltia cancergena');
INSERT INTO "TIPUSMALALTIES" ("CODIMALALTIA", "DESCRIPCIO") 
  VALUES (5, 'Malaltia degenerativa del sistema nervis');

-- ---------------------------------------------------------------------------
-- Crear la taula de registres d'estat de salut.
-- ---------------------------------------------------------------------------
CREATE TABLE "ESTATSALUT" 
(
  "DATA" DATE NOT NULL,
  "CODIPAIS" CHAR(2 CHAR) NOT NULL, 
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  "CODIMALALTIA" INTEGER NOT NULL,
  CONSTRAINT "PK_ESTATSALUT" 
    PRIMARY KEY ("DATA", "CODIPAIS", "IDCIUTADA", "CODIMALALTIA"),
  CONSTRAINT "FK_ESTATSALUT_CIUTADANS" 
    FOREIGN KEY ("CODIPAIS","IDCIUTADA")
    REFERENCES "CIUTADANS" ("CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_ESTATSALUT_TIPUSMALALTIES" 
    FOREIGN KEY ("CODIMALALTIA")
    REFERENCES "TIPUSMALALTIES" ("CODIMALALTIA")
);

-- ---------------------------------------------------------------------------
-- Crear la taula de tipus d'estadstiques.
-- ---------------------------------------------------------------------------
CREATE TABLE "TIPUSESTADISTIQUES"
(
  "CODIESTADISTICA" INTEGER NOT NULL PRIMARY KEY,
  "DESCRIPCIO" VARCHAR2(200 CHAR) NOT NULL 
);

-- Carregar les dades dels pasos a la taula TipusEstadistiques.
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (1, 'El nombre de ciutadans que han fet servir el sistema durant aquell any (s a dir que com a mnim hagin enviat informaci d''activitat d''un dia en aquell any).');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (2, 'La suma de totes les calories gastades pels ciutadans del pas durant aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (3, 'La suma de totes les passes donades pels ciutadans del pas durant aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (4, 'La suma de totes les hores de poca activitat dels ciutadans del pas durant aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (5, 'El nombre de ciutadans als quals se''ls ha diagnosticat una malaltia cardaca en aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (6, 'El nombre de ciutadans als quals se''ls ha diagnosticat una malaltia cancergena en aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (7, 'El nombre de ciutadans als quals se''ls ha diagnosticat una malaltia degenerativa del sistema nervis en aquell any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (8, 'La suma total de malalties lleus diagnosticades durant aquell l''any.');
INSERT INTO "TIPUSESTADISTIQUES" ("CODIESTADISTICA", "DESCRIPCIO") 
  VALUES (9, 'L''identificador de la persona que ms calories ha cremat.');

-- ---------------------------------------------------------------------------
-- Crear la taula de valors resultats de les estadstiques.
-- ---------------------------------------------------------------------------
CREATE TABLE "ESTADISTICAVALORS"
(
  "ANY" NUMBER(4,0) NOT NULL,
  "CODIESTADISTICA" INTEGER NOT NULL,
  "CODIPAIS" CHAR(2 CHAR) NOT NULL, 
  "VALOR" INTEGER NOT NULL,
  CONSTRAINT "PK_ESTADISTICAVALORS" 
    PRIMARY KEY ("ANY", "CODIESTADISTICA", "CODIPAIS"),
  CONSTRAINT "FK_ESTADVALORS_CODIPAIS" 
    FOREIGN KEY ("CODIPAIS")
    REFERENCES "PAISOS" ("CODIPAIS"),
  CONSTRAINT "FK_ESTADVALORS_CODIESTADISTICA" 
    FOREIGN KEY ("CODIESTADISTICA")
    REFERENCES "TIPUSESTADISTIQUES" ("CODIESTADISTICA")
);

-- ---------------------------------------------------------------------------
-- Crear la taula de valors resultats de la estadstica nmero 9.
-- ---------------------------------------------------------------------------
CREATE TABLE "ESTADISTICACALORIES"
(
  "ANY" NUMBER(4,0) NOT NULL,
  "CODIESTADISTICA" INTEGER NOT NULL,
  "CODIPAISE" CHAR(2 CHAR) NOT NULL,
  "CODIPAISC" CHAR(2 CHAR) NOT NULL,
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  CONSTRAINT "PK_ESTADISTICACALORIES"
    PRIMARY KEY ("ANY", "CODIESTADISTICA", "CODIPAISE"),
  CONSTRAINT "FK_ESTADCALORIES_PAISOS" 
    FOREIGN KEY ("CODIPAISE")
    REFERENCES "PAISOS" ("CODIPAIS"),
  CONSTRAINT "FK_ESTADCALORIES_CIUTADANS" 
    FOREIGN KEY ("CODIPAISC", "IDCIUTADA")
    REFERENCES "CIUTADANS" ("CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_ESTADCALORIES_CODIEST" 
    FOREIGN KEY ("CODIESTADISTICA")
    REFERENCES "TIPUSESTADISTIQUES" ("CODIESTADISTICA")
);

-- ---------------------------------------------------------------------------
-- Crear la taula de rnking de ciutadans que han gastat calories.
-- ---------------------------------------------------------------------------
CREATE TABLE "RANKINGCALORIES"
(
  "ANY" NUMBER(4,0) NOT NULL,
  "CODIPAISE" CHAR(2 CHAR) NOT NULL, 
  "CODIPAISC" CHAR(2 CHAR) NOT NULL, 
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  "CALORIES" INTEGER NOT NULL,
  CONSTRAINT "PK_RANKINGCALORIES"
    PRIMARY KEY ("ANY", "CODIPAISE", "CODIPAISC", "IDCIUTADA"),
  CONSTRAINT "FK_RANKINGCALORIES_CIUTADANS" 
    FOREIGN KEY ("CODIPAISC", "IDCIUTADA")
    REFERENCES "CIUTADANS" ("CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_RANKINGCALORIES_PAISOS" 
    FOREIGN KEY ("CODIPAISE")
    REFERENCES "PAISOS" ("CODIPAIS")
);

-- ndex alternatiu per buscar rpidament.
CREATE UNIQUE INDEX "IDX_RANKING_CALORIES"
  ON system."RANKINGCALORIES" 
    ("ANY" ASC, "CODIPAISE" ASC, "CALORIES" DESC, "CODIPAISC" ASC, "IDCIUTADA" ASC);

-- ---------------------------------------------------------------------------
-- Crear la taula de s del sistema per part dels ciutadans.
-- ---------------------------------------------------------------------------
CREATE TABLE "USSISTEMA"
(
  "ANY" NUMBER(4,0) NOT NULL,
  "CODIPAIS" CHAR(2 CHAR) NOT NULL, 
  "IDCIUTADA" VARCHAR2(20 CHAR) NOT NULL, 
  CONSTRAINT "PK_USSISTEMA"
    PRIMARY KEY ("ANY", "CODIPAIS", "IDCIUTADA"),
  CONSTRAINT "FK_USSISTEMA_CIUTADANS" 
    FOREIGN KEY ("CODIPAIS", "IDCIUTADA")
    REFERENCES "CIUTADANS" ("CODIPAIS", "IDCIUTADA")
);

-- ---------------------------------------------------------------------------
-- Crear la taula de registre de crides als procediments emmagatzemats pblics.
-- ---------------------------------------------------------------------------
CREATE TABLE "LOGPROCEDIMENTS"
(
  "NUMORDRE" INTEGER NOT NULL PRIMARY KEY,
  "DATAHORA" TIMESTAMP NOT NULL,
  "NOMPROCEDIMENT" VARCHAR2(50 CHAR) NOT NULL,
  "TIPUS" CHAR(1 CHAR) NOT NULL, 
  "PARAMETRES" VARCHAR2(200 CHAR) NOT NULL, 
  "RESULTAT" VARCHAR2(10 CHAR) NOT NULL,
  CONSTRAINT "CK_LOGPROCEDIMENTS_TIPUS" 
    CHECK ("TIPUS" = 'E' OR "TIPUS" = 'S')
);

-- Sequncia per a la numeraci automtica de registres del log.
CREATE SEQUENCE "seqLogProcediments" INCREMENT BY 1 START WITH 1;

-- Disaparador per l'assinaci automtica de la seqncia del log.
CREATE OR REPLACE TRIGGER "trgNewLogProcediments"
BEFORE INSERT ON system."LOGPROCEDIMENTS"
FOR EACH ROW
BEGIN
  SELECT "seqLogProcediments".NEXTVAL INTO :NEW.NumOrdre FROM DUAL;
END trgNewLogProcediments;
/

-- ***************************************************************************
-- Final del script.
-- ***************************************************************************
