-- ***************************************************************************
-- Script de testing de la base de dades UOC_SGDB
-- Autor: Miguel ngel Bueno Snchez
-- TFC BBDD Relacionals
-- 2014-2015-1
-- ***************************************************************************
set serveroutput on;

-- --------------------------------------------------------
-- Definir variables.
-- --------------------------------------------------------
DECLARE
  RSP VARCHAR2(10 CHAR);
  missatge VARCHAR2(256 CHAR);
  quants INTEGER;
  malaltia INTEGER;
  pais CHAR(2);
  ciutada VARCHAR2(20);
  pAny NUMBER;
  calories INTEGER;
  dia INTEGER;
  mes INTEGER;
  hores INTEGER;
  pases INTEGER;
  tests_date DATE;
BEGIN

RSP := '';
missatge := '';
tests_date := TO_DATE('12-01-2015','DD-MM-YYYY');

-- --------------------------------------------------------
-- Capalera de l'informe.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE( LPAD( '=', 80, '=' ));
DBMS_OUTPUT.PUT_LINE(' TESTS DE LA BASE DE DADES UOC_SGDB');
DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE(' Data d''execuci: ' || current_date);
DBMS_OUTPUT.PUT_LINE(' Data pels tests: ' || tests_date);
DBMS_OUTPUT.PUT_LINE( LPAD( '=', 80, '=' ));

-- --------------------------------------------------------
-- Esborrar les dades existents.
-- --------------------------------------------------------

delete from rankingcalories;
delete from estadisticacalories;
delete from estadisticavalors;
delete from ussistema;
delete from estatsalut;
delete from activitatfisica;
delete from ciutadans;
delete from logprocediments;

-- --------------------------------------------------------
-- Tests de taules.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS DE TAULES');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Taula de pasos de la Uni Europea ...');
for country in ( select * from TABLE(GetCountriesList) )
loop
  DBMS_OUTPUT.PUT_LINE(country."COUNTRYCODE" || ' - ' || country."NAME");
end loop;
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Nmero de registres de la taula de pasos de la Uni Europea ...');
select count(*) into quants 
  from TABLE(GetCountriesList);
if (quants <> 28) then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Taula de pasos de la Uni Europea, sense el codi per la Uni Europea ...');
select count(*) into quants 
  from TABLE(GetCountriesList)
  where "COUNTRYCODE" = 'UE';
if (quants <> 0) then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Taula de malalties suportades pel sistema ...');
for disease in ( select * from TABLE(GetDiseasesList) )
loop
  DBMS_OUTPUT.PUT_LINE(disease."DISEASECODE" || ' - ' || disease."DESCRIPTION");
end loop;
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Nmero de registres de la taula de malalties suportades pel sistema ...');
select count(*) into quants 
  from TABLE(GetDiseasesList);
if (quants <> 5) then
  goto final_error;
end if;

-- --------------------------------------------------------
-- Tests de ciutadans.
-- --------------------------------------------------------
DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS DELS CIUTADANS');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('TESTS DE CREACI DE CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Creant ciutad ES 1 ...');
CreateCitizen('ES', '1', 'Juan', 'Lpez', 'H', 'juanlopez@email.com', tests_date - 30 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad ES 2 ...');
CreateCitizen('ES', '2', 'Pio', 'Ferndez', 'H', 'piolopez@email.com', tests_date - 30 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad ES 3 ...');
CreateCitizen('ES', '3', 'Maria', 'Gmis', 'D', 'mariagomis@email.com', tests_date - 40 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad ES 4 ...');
CreateCitizen('ES', '4', 'Pepa', 'Snchez', 'D', 'pepasanchez@email.com', tests_date - 35 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad FR 1 ...');
CreateCitizen('FR', '1', 'Fraoise', 'Gibernau', 'H', 'fracoisegibernau@email.com', tests_date - 20 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad FR 2 ...');
CreateCitizen('FR', '2', 'Alain', 'Martin', 'H', 'alainmartin@email.com', tests_date - 25 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad DK 3 ...');
CreateCitizen('DK', '3', 'Mery', 'Danks', 'D', 'merydanks@email.com', tests_date - 30 * 365,RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad sense nom ...');
CreateCitizen('IT', '1', '', 'Rodrigo', 'D', 'rodrigo@email.com', tests_date - 35 * 365,RSP);
if (RSP <> 'ERROR:003') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad sense cognoms ...');
CreateCitizen('IT', '2', 'Esteban', '', 'D', 'esteban@email.com', tests_date - 30 * 365,RSP);
if (RSP <> 'ERROR:004') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad de sexe no valid DK 4 ...');
CreateCitizen('PT', '1', 'Mariana', 'Lagos', 'X', 'marianlagos@email.com', tests_date - 25 * 365,RSP);
if (RSP <> 'ERROR:005') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad repetit DK 3 ...');
CreateCitizen('DK', '3', 'John', 'Johanson', 'H', 'johnjohanson@email.com', tests_date - 45 * 365,RSP);
if (RSP <> 'ERROR:006') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Creant ciutad de pas no vlid XX 1 ...');
CreateCitizen('XX', '1', 'Sheila', 'Fernndez', 'D', 'sheilafernandez@email.com', tests_date - 80 * 365,RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ELIMINACI DE CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Eliminar el ciudad ES 4 ...');
RemoveCitizen('ES', '4', RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Eliminar el ciudad inexistent ES 4 ...');
RemoveCitizen('ES', '10', RSP);
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Eliminar el ciudad de pas no vlid XX 1 ...');
RemoveCitizen('XX', '1', RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ACTUALITZACI DE CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Actualitzar el ciudad de pas no vlid XX 1 ...');
UpdateCitizen('XX', '1', 'Frank', 'Dens', 'H', 'frankdens@email.com', tests_date - 30 * 365, RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Actualitzar el ciudad no existent DE 1 ...');
UpdateCitizen('DE', '1', 'Frank', 'Dens', 'H', 'frankdens@email.com', tests_date - 30 * 365, RSP);
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Actualitzar el ciudad vlid ...');
UpdateCitizen('DK', '3', 'Joan', 'Danks', 'D', 'joandanks@email.com', tests_date - 25 * 365, RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS DE CONTADORS DE REGISTRES DE CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Verificar nombre de ciutadans totals ...');
select count(*) into quants from CIUTADANS;
if (quants <> 7) then
  RSP := '';
  missatge := 'Nmero erroni de ciutadans totals a la base de dades.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Verificar nombre de ciutadans donats d''alta...');
select count(*) into quants from CIUTADANS where "ACTIU" = 'S';
if (quants <> 6) then
  RSP := '';
  missatge := 'Nmero erroni de ciutadans donats d''alta a la base de dades.';
  goto final_error;
end if;

-- --------------------------------------------------------
--- Tests d'activitat fsica.
-- --------------------------------------------------------
DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS D''ACTIVITAT FISICA');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('TESTS DE CREACI DE REGISTRES D''ACTIVITAT FISICA:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 1 ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 100, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 1 el mateix dia...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 200, 2500, 10, RSP); 
if (RSP <> 'ERROR:009') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 1 un altre dia...');
AddPhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '1', 200, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 1 un altre any...');
AddPhysicalActivityData( TO_DATE('01-01-2015', 'DD-MM-YYYY'), 'ES', '1', 200, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 2 ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', 100, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 3 ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '3', 100, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad ES 3 un altre dia...');
AddPhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '3', 100, 2500, 10, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad FR 1 ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'FR', '1', 300, 3500, 8, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica del ciudad FR 2 ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'FR', '2', 400, 4000, 9, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica d''un ciutada inexistent...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'PT', '1', 300, 3500, 8, RSP); 
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica d''un ciutada donat de baixa...');
AddPhysicalActivityData( TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '4', 300, 3500, 8, RSP); 
if (RSP <> 'ERROR:008') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Afegir dades d''activitat fsica amb hores d''inactivitat incorrectes...');
AddPhysicalActivityData( TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '4', 300, 3500, 25, RSP); 
if (RSP <> 'ERROR:016') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ELIMINACI DE REGISTRES D''ACTIVITAT FISICA:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Eliminar dades d''activitat fsica del ciutada ES 3 ...');
RemovePhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '3', RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Eliminar un registre de dades d''activitat fsica inexistent ...');
RemovePhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '3', RSP);
if (RSP <> 'ERROR:010') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Eliminar dades d''activitat fsica d''un ciutada inexistent...');
RemovePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'PT', '1', RSP); 
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Eliminar dades d''activitat fsica d''un ciutada donat de baixa...');
RemovePhysicalActivityData( TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '4', RSP); 
if (RSP <> 'ERROR:008') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ACTUALITZACI DE REGISTRES D''ACTIVITAT FISICA:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Actualitzar dades d''activitat fsica del ciutada ES 1 ...');
UpdatePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 500, 6000, 11, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Actualitzar dades d''activitat fsica d''un ciutada inexistent...');
UpdatePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'PT', '1', 1, 1, 1, RSP); 
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Actualitzar dades d''activitat fsica d''un ciutada donat de baixa...');
UpdatePhysicalActivityData( TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '4', 0, 0, 0, RSP); 
if (RSP <> 'ERROR:008') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Actualitzar dades d''activitat fsica amb hores d''inactivitat incorrectes...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 300, 3500, 25, RSP); 
if (RSP <> 'ERROR:016') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS DE CONTADORS DE REGISTRES D''ACTIVITAT FSICA:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Verificar nombre de registres donats d''alta ...');
select count(*) into quants from ACTIVITATFISICA;
if (quants <> 7) then
  RSP := '';
  missatge := 'Nmero erroni de registres d''activitat fsica donats d''alta a la base de dades.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Verificar nombre de registres de pasos diferents ...');
select count(distinct codipais) into quants from ACTIVITATFISICA;
if (quants <> 2) then
  RSP := '';
  missatge := 'Nmero erroni de registres d''activitat fsica de pasos diferents.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Verificar suma de passes ...');
select sum(passes) into quants from ACTIVITATFISICA;
if (quants <> 1800) then
  RSP := '';
  missatge := 'Nmero erroni de passes als registres d''activitat fsica.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Verificar suma de calories ...');
select sum(calories) into quants from ACTIVITATFISICA;
if (quants <> 23500) then
  RSP := '';
  missatge := 'Nmero erroni de calories als registres d''activitat fsica.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Verificar suma d''hores d''inactivitat ...');
select sum(horesinactivitat) into quants from ACTIVITATFISICA;
if (quants <> 68) then
  RSP := '';
  missatge := 'Nmero erroni d''hores d''inactivitat als registres d''activitat fsica.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ESTADSTIQUES DE REGISTRES D''ACTIVITAT FSICA:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Estadstiques : any incorrecte ...');
quants := GetNumberOfCitizens(0, 'ES', RSP);
if (RSP <> 'ERROR:013') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Estadstiques : pas incorrecte ...');
quants := GetNumberOfCitizens(2014, 'XX', RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Estadstica 1 : nmero de ciutadans que han aportat dades al sistema ...');
quants := GetNumberOfCitizens(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 ES.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'ES', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 ES.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 FR.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 FR.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 5) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 UE.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 2 : suma de calories gastades pels ciutadans d''un pas en un any ...');
quants := GetExpendedCalories(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 13500) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2014 ES.';
  goto final_error;
end if;
quants := GetExpendedCalories(2015, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2500) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2015 ES.';
  goto final_error;
end if;
quants := GetExpendedCalories(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 7500) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2014 FR.';
  goto final_error;
end if;
quants := GetExpendedCalories(2015, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2015 FR.';
  goto final_error;
end if;
quants := GetExpendedCalories(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 21000) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2014 UE.';
  goto final_error;
end if;
quants := GetExpendedCalories(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 2500) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 2 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 3 : suma de passes realitzades pels ciutadans d''un pas en un any ...');
quants := GetStepsWalked(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 900) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2014 ES.';
  goto final_error;
end if;
quants := GetStepsWalked(2015, 'ES', RSP);
if (RSP <> 'OK' or quants <> 200) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2015 ES.';
  goto final_error;
end if;
quants := GetStepsWalked(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 700) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2014 FR.';
  goto final_error;
end if;
quants := GetStepsWalked(2015, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2015 FR.';
  goto final_error;
end if;
quants := GetStepsWalked(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 1600) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2014 UE.';
  goto final_error;
end if;
quants := GetStepsWalked(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 200) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 3 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 4 : suma d''hores d''inactivitat dels ciutadans d''un pas en un any ...');
quants := GetDowntime(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 41) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2014 ES.';
  goto final_error;
end if;
quants := GetDowntime(2015, 'ES', RSP);
if (RSP <> 'OK' or quants <> 10) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2015 ES.';
  goto final_error;
end if;
quants := GetDowntime(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 17) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2014 FR.';
  goto final_error;
end if;
quants := GetDowntime(2015, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2015 FR.';
  goto final_error;
end if;
quants := GetDowntime(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 58) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2014 UE.';
  goto final_error;
end if;
quants := GetDowntime(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 10) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 4 2015 UE.';
  goto final_error;
end if;

-- --------------------------------------------------------
--- Tests d'estat de salut.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS D''ESTAT DE SALUT');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('TESTS DE CREACI DE REGISTRES D''ESTAT DE SALUT DELS CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Malalties lleus ...');

DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 ...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 el mateix dia ...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, RSP); 
if (RSP <> 'ERROR:011') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 un altre dia...');
AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad FR 2 ...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'FR', '2', 2, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut d''un ciutad donat de baixa...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '4', 2, RSP); 
if (RSP <> 'ERROR:008') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut d''un ciutad inexistent...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '10', 2, RSP); 
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad DK 3 ...');
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'DK', '3', 2, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;

-- Crear les mateixes dades per les malalties 3, 4 i 5, haurien de donar com a resultat les mateixes estadstiques.
for malaltia in 3..5
loop 
    case malaltia
    when 3 then
      DBMS_OUTPUT.PUT_LINE('- Malalties cardaques ...');

    when 4 then
      DBMS_OUTPUT.PUT_LINE('- Malalties cancergenes ...');

    when 5 then
      DBMS_OUTPUT.PUT_LINE('- Malalties degeneratives del sistema nervis ...');

    end case;

    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 2 ...');
    AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 2 un altre dia ...');
    AddHealthData(TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 el mateix dia que la malaltia lleu ...');
    AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 el mateix dia ...');
    AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', malaltia, RSP); 
    if (RSP <> 'ERROR:011') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 1 un altre dia ...');
    AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad ES 3 ...');
    AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '3', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad FR 2 ...');
    AddHealthData(TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'FR', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad FR 2 un altre dia ...');
    AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'FR', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad DK 3 un altre dia ...');
    AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'DK', '3', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Afegir dades d''estat de salut del ciutad DK 3 un altre any ...');
    AddHealthData(TO_DATE('01-01-2015', 'DD-MM-YYYY'), 'DK', '3', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
end loop;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ELIMINACI DE REGISTRES D''ESTAT DE SALUT DELS CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

-- Eliminar les mateixes dades per les malalties 3, 4 i 5, haurien de donar com a resultat les mateixes estadstiques.
for malaltia in 3..5
loop 
    case malaltia
    when 3 then
      DBMS_OUTPUT.PUT_LINE('- Malalties cardaques ...');

    when 4 then
      DBMS_OUTPUT.PUT_LINE('- Malalties cancergenes ...');

    when 5 then
      DBMS_OUTPUT.PUT_LINE('- Malalties degeneratives del sistema nervis ...');

    end case;

    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad ES 2 ...');
    RemoveHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad ES 2 el mateix dia i malaltia ...');
    RemoveHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', malaltia, RSP); 
    if (RSP <> 'ERROR:012') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad FR 2 ...');
    RemoveHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'FR', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad ES 1...');
    RemoveHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad FR 2 i deixar-lo sense malalties...');
    RemoveHealthData(TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'FR', '2', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
    DBMS_OUTPUT.PUT_LINE('  - Eliminar dades d''estat de salut del ciutad ES 3...');
    RemoveHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '3', malaltia, RSP); 
    if (RSP <> 'OK') then
      goto final_error;
    end if;
end loop;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS DE CONTADORS DE REGISTRES D''ESTAT DE SALUT:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Verificar nombre de registres donats d''alta al pas ES...');
select count(*) into quants from ESTATSALUT where CODIPAIS = 'ES';
if (quants <> 8) then
  RSP := '';
  missatge := 'Nmero erroni de registres d''estat de salut del pas ES.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Verificar nombre de registres donats d''alta al pas FR...');
select count(*) into quants from ESTATSALUT where CODIPAIS = 'FR';
if (quants <> 1) then
  RSP := '';
  missatge := 'Nmero erroni de registres d''estat de salut del pas FR.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Verificar nombre de registres donats d''alta al pas DK...');
select count(*) into quants from ESTATSALUT where CODIPAIS = 'DK';
if (quants <> 7) then
  RSP := '';
  missatge := 'Nmero erroni de registres d''estat de salut del pas DK.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS D''ESTADSTIQUES DE REGISTRES D''ESTAT DE SALUT:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Estadstica 5 : nmero de ciutadans amb malaltia cardaca en un any ...');
quants := GetHeartDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 ES.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 FR.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 DK.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 UE.';
  goto final_error;
end if;
quants := GetHeartDiseases(2015, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2015 DK.';
  goto final_error;
end if;
quants := GetHeartDiseases(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 6 : nmero de ciutadans amb malaltia cancergena en un any ...');
quants := GetCancerDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 ES.';
  goto final_error;
end if;
quants := GetCancerDiseases(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 FR.';
  goto final_error;
end if;
quants := GetCancerDiseases(2014, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 DK.';
  goto final_error;
end if;
quants := GetCancerDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 UE.';
  goto final_error;
end if;
quants := GetCancerDiseases(2015, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2015 DK.';
  goto final_error;
end if;
quants := GetCancerDiseases(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 7 : nmero de ciutadans amb malaltia degenerativa en un any ...');
quants := GetDegenerativeDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2014 ES.';
  goto final_error;
end if;
quants := GetDegenerativeDiseases(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2014 FR.';
  goto final_error;
end if;
quants := GetDegenerativeDiseases(2014, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2014 DK.';
  goto final_error;
end if;
quants := GetDegenerativeDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2014 UE.';
  goto final_error;
end if;
quants := GetDegenerativeDiseases(2015, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2015 DK.';
  goto final_error;
end if;
quants := GetDegenerativeDiseases(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 7 2015 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Estadstica 8 : nmero total de malalties lleus ...');
quants := GetMinorIllness(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 ES.';
  goto final_error;
end if;
quants := GetMinorIllness(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 FR.';
  goto final_error;
end if;
quants := GetMinorIllness(2014, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 DK.';
  goto final_error;
end if;
quants := GetMinorIllness(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 4) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('TESTS DE MODIFICACI DE REGISTRES D''ESTAT DE SALUT DELS CIUTADANS:');
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Modificar una malaltia lleu per una altre de cardaca en una data en la qual el ciutad ja en t.');
UpdateHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, 3, RSP); 
if (RSP <> 'ERROR:011') then
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Modificar una malaltia lleu per una altre de cardaca.');
UpdateHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, 3, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;
quants := GetMinorIllness(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 ES.';
  goto final_error;
end if;
quants := GetMinorIllness(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 8 2014 UE.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 ES.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 UE.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Modificar una malaltia cardaca per una altre de cancergena.');
UpdateHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', 3, 4, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 ES.';
  goto final_error;
end if;
quants := GetHeartDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 5 2014 UE.';
  goto final_error;
end if;
quants := GetCancerDiseases(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 ES.';
  goto final_error;
end if;
quants := GetCancerDiseases(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 6 2014 UE.';
  goto final_error;
end if;

-- --------------------------------------------------------
-- Tests d'estadstiques d's del sistema..
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS D''ESTADSTIQUES D''S DEL SISTEMA');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Estadstica 1 : nmero de ciutadans que han aportat dades al sistema ...');
quants := GetNumberOfCitizens(2014, 'ES', RSP);
if (RSP <> 'OK' or quants <> 3) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 ES.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'ES', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 ES.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2014, 'FR', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 FR.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'FR', RSP);
if (RSP <> 'OK' or quants <> 0) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 FR.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2014, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 DK.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'DK', RSP);
if (RSP <> 'OK' or quants <> 1) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 DK.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2014, 'UE', RSP);
if (RSP <> 'OK' or quants <> 6) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2014 UE.';
  goto final_error;
end if;
quants := GetNumberOfCitizens(2015, 'UE', RSP);
if (RSP <> 'OK' or quants <> 2) then
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 1 2015 UE.';
  goto final_error;
end if;

-- --------------------------------------------------------
-- Tests del llistat d'estat de salut dels ciutadans.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS DEL LLISTAT D''ESTAT DE SALUT DELS CIUTADANS');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

-- Afegir dades per completar el llistat d'estat de salut dels ciutadans.
AddHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 1, RSP); 
AddHealthData(TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '1', 2, RSP); 
AddHealthData(TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '1', 1, RSP); 
AddHealthData(TO_DATE('01-01-2013', 'DD-MM-YYYY'), 'ES', '1', 3, RSP); 
AddHealthData(TO_DATE('01-01-2011', 'DD-MM-YYYY'), 'ES', '1', 3, RSP); 
AddHealthData(TO_DATE('01-01-2015', 'DD-MM-YYYY'), 'ES', '1', 1, RSP); 
AddHealthData(TO_DATE('01-01-2010', 'DD-MM-YYYY'), 'ES', '1', 4, RSP); 
RemoveHealthData(TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '2', 4, RSP); 
RemoveHealthData(TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 5, RSP); 

AddPhysicalActivityData( TO_DATE('04-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 2500, 0, RSP); 
AddPhysicalActivityData( TO_DATE('01-01-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 9000, 0, RSP); 
AddPhysicalActivityData( TO_DATE('31-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 7000, 0, RSP); 
AddPhysicalActivityData( TO_DATE('05-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 2500, 0, RSP); 
AddPhysicalActivityData( TO_DATE('06-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 3000, 0, RSP); 
AddPhysicalActivityData( TO_DATE('07-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 5000, 0, RSP); 
AddPhysicalActivityData( TO_DATE('07-12-2014', 'DD-MM-YYYY'), 'ES', '2', 0, 7500, 0, RSP); 

-- Aquest llistat es pot veure per pantalla fent:
ShowCitizensHealthStateList(2014, 'ES', 30, 6000, RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

-- Verificar parmetres del llistat.
ShowCitizensHealthStateList(0, 'ES', 30, 6000, RSP);
if (RSP <> 'ERROR:013') then
  goto final_error;
end if;
ShowCitizensHealthStateList(2014, 'UE', 30, 6000, RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;
ShowCitizensHealthStateList(2014, 'ES', 151, 6000, RSP);
if (RSP <> 'ERROR:014') then
  goto final_error;
end if;
ShowCitizensHealthStateList(2014, 'UE', 30, -1, RSP);
if (RSP <> 'ERROR:015') then
  goto final_error;
end if;

-- Verificar nmero de ciutadans.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys del pas ''ES''...');
select count(*) into quants 
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 5000)); 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES''.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys del pas ''ES''...');
select count(*) into quants 
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 5001)); 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES''.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 40 anys del pas ''ES''...');
select count(*) into quants 
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 40, 6000)); 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 40 ''ES''.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 40 anys del pas ''ES'' amb baix llindar de calories...');
select count(*) into quants 
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 40, 2000)); 
if (quants <> 0) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 40 ''ES'' amb baix llindar de calories.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 25 anys del pas ''ES''...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 25, 4000)); 
if (quants <> 0) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 25 ''ES''.';
  goto final_error;
end if;

-- Verificar any de malaltes cardaques.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' sense malalties cardaques...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCARDIACA" is null; 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' sense malaltia cardaca.';
  goto final_error;
end if;
select "IDCIUTADA" into ciutada
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCARDIACA" is null; 
if (ciutada <> '2') then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' sense malaltia cardaca.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb malalties cardaques...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCARDIACA" is not null; 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb malaltia cardaca.';
  goto final_error;
end if;
select "IDCIUTADA", "ANYCARDIACA" into ciutada, pAny
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCARDIACA" is not null; 
if (ciutada <> '1' or pAny <> 2010) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb malaltia cardaca.';
  goto final_error;
end if;

-- Verificar any de malaltes cancergenes.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' sense malalties cancergena...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCANCER" is null; 
if (quants <> 0) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' sense malaltia cancergena.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb malalties cancergena...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYCANCER" is not null; 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb malaltia cancergena.';
  goto final_error;
end if;

-- Verificar any de malaltes degeneratives.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' sense malalties degenerativa...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYDEGENERATIVA" is null; 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' sense malaltia degenerativa.';
  goto final_error;
end if;
select "IDCIUTADA" into ciutada
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYDEGENERATIVA" is null; 
if (ciutada <> '1') then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' sense malaltia degenerativa.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb malalties degenerativa...');
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYDEGENERATIVA" is not null; 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb malaltia degenerativa.';
  goto final_error;
end if;
select "IDCIUTADA", "ANYDEGENERATIVA" into ciutada, pAny
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "ANYDEGENERATIVA" is not null; 
if (ciutada <> '2' or pAny <> 2014) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb malaltia degenerativa.';
  goto final_error;
end if;

-- Verificar mitjana de calories gastades
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb nombre mitj de calories gastades...');
select "MITJACALORIES" into calories
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "IDCIUTADA" = '1'; 
if (calories <> 4688) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' ''1'' amb nombre mitj de calories gastades.';
  goto final_error;
end if;
select "MITJACALORIES" into calories
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "IDCIUTADA" = '2'; 
if (calories <> 5000) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' ''2'' amb nombre mitj de calories gastades.';
  goto final_error;
end if;
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 5000))
  where "MITJACALORIES" > 0; 
if (quants <> 1) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb baix llindar de calories gastades.';
  goto final_error;
end if;
select count(*) into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 2500))
  where "MITJACALORIES" > 0; 
if (quants <> 0) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' amb molt baix llindar de calories gastades.';
  goto final_error;
end if;

-- Verificar nmero de visites al metge.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb visites al metge...');
select "VISITESMETGE" into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "IDCIUTADA" = '1'; 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' ''1'' amb visites al metge.';
  goto final_error;
end if;

-- Verificar nmero de malalties lleus.
DBMS_OUTPUT.PUT_LINE('- Ciutadans de 30 anys de ''ES'' amb malalties lleus...');
select "MALALTIESLLEUS" into quants
  from TABLE(funcLlistatSalutCiutadans(2014, 'ES', 30, 6000))
  where "IDCIUTADA" = '1'; 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat de ciutadans 30 ''ES'' ''1'' amb malalties lleus.';
  goto final_error;
end if;

-- --------------------------------------------------------
-- Tests del llistat d'activitat fsica mensual dels ciutadans.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS DEL LLISTAT DE L''ACTIVITAT FSICA MENSUAL D''UN CIUTAD');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

-- Esborrar les dades existents.
delete from rankingcalories;
delete from estadisticacalories;
delete from estadisticavalors;
delete from ussistema;
delete from estatsalut;
delete from activitatfisica;
delete from logprocediments;

-- Generar de manera automtica dades pel llistat de l'activitat fsica mensual d'un ciutad.
calories := 0;
pases := 0;
hores := 0;
for mes in 1..12
loop
  for dia in 1..28
  loop 
    pases := pases + 50;
    if (pases > 2000) then
      pases := 0;
    end if;
    calories := calories + 1000;
    if (calories > 10000) then
      calories := 0;
    end if;
    hores := hores + 1;
    if (hores > 15) then
      hores := 0;
    end if;
    
    AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'ES', '1', pases, calories, hores, RSP); 
  end loop;
end loop;

-- Aquest llistat es pot veure per pantalla fent:
ShowCitizenMonthlyActivityList(2014, 'ES', '1', RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;

-- Verificar parmetres del llistat.
ShowCitizenMonthlyActivityList(0, 'ES', '1', RSP);
if (RSP <> 'ERROR:013') then
  goto final_error;
end if;
ShowCitizenMonthlyActivityList(2014, 'UE', '1', RSP);
if (RSP <> 'ERROR:001') then
  goto final_error;
end if;
ShowCitizenMonthlyActivityList(2014, 'ES', '', RSP);
if (RSP <> 'ERROR:002') then
  goto final_error;
end if;
ShowCitizenMonthlyActivityList(2014, 'ES', '0', RSP);
if (RSP <> 'ERROR:007') then
  goto final_error;
end if;

-- Verificar nmero de registres del llistat.
DBMS_OUTPUT.PUT_LINE('- Nmero de registres del llistat...');
select count(*) into quants 
  from TABLE(funcActFisicaMensualCiutada(2014, 'ES', '1')); 
if (quants <> 12) then
  RSP := '';
  missatge := 'Resultat errni del llistat d''activitat fsica del ciutad ES 1.';
  goto final_error;
end if;
quants := 0;

-- Verificar els registres del llistat.
DBMS_OUTPUT.PUT_LINE('- Nmero de passes, calories i hores del ciutad ES 1 al Gener...');
select "PASSES", "MITJACALORIES", "HORESINACTIVITAT" 
  into pases, calories, hores 
  from TABLE(funcActFisicaMensualCiutada(2014, 'ES', '1'))
  where "NOMMES" = 'Gener'; 
if (pases <> 20300) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni del nmero de passes del ciutad ES 1 al Gener.';
  goto final_error;
end if;
if (calories <> 4679) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni del nmero de calories del ciutad ES 1 al Gener.';
  goto final_error;
end if;
if (hores <> 198) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni del nmero d''hores del ciutad ES 1 al Gener.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero de passes, calories i hores del ciutad ES 1 al Juliol...');
select "PASSES", "MITJACALORIES", "HORESINACTIVITAT" 
  into pases, calories, hores 
  from TABLE(funcActFisicaMensualCiutada(2014, 'ES', '1'))
  where "NOMMES" = 'Juliol'; 
if (pases <> 25900) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni del nmero de passes del ciutad ES 1 al Juliol.';
  goto final_error;
end if;
if (calories <> 5321) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni del nmero de calories del ciutad ES 1 al Juliol.';
  goto final_error;
end if;
if (hores <> 214) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni del nmero d''hores del ciutad ES 1 al Juliol.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero de passes, calories i hores del ciutad ES 1 al Dessembre...');
select "PASSES", "MITJACALORIES", "HORESINACTIVITAT" 
  into pases, calories, hores 
  from TABLE(funcActFisicaMensualCiutada(2014, 'ES', '1'))
  where "NOMMES" = 'Dessembre'; 
if (pases <> 31250) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni del nmero de passes del ciutad ES 1 al Dessembre.';
  goto final_error;
end if;
if (calories <> 4679) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni del nmero de calories del ciutad ES 1 al Dessembre.';
  goto final_error;
end if;
if (hores <> 230) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni del nmero d''hores del ciutad ES 1 al Dessembre.';
  goto final_error;
end if;

-- --------------------------------------------------------
-- Tests del llistat de l'activitat fsica als pasos.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS DEL LLISTAT DE L''ACTIVITAT FSICA DE TOTS ELS PASOS');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

-- Verificar parmetres del llistat.
ShowCountryActivityList(0, 30, RSP);
if (RSP <> 'ERROR:013') then
  goto final_error;
end if;
ShowCountryActivityList(2014, -1, RSP);
if (RSP <> 'ERROR:014') then
  goto final_error;
end if;
ShowCountryActivityList(2014, 160, RSP);
if (RSP <> 'ERROR:014') then
  goto final_error;
end if;

-- Generar de manera automtica dades pel llistat de l'activitat fsica de diferents pasos.
calories := 0;
pases := 0;
hores := 0;
for quants in 1..5 loop
  for mes in 1..12 loop
    for dia in 1..28 loop 
      pases := pases + 50;
      if (pases > 2000) then
        pases := 0;
      end if;
      calories := calories + 500;
      if (calories > 10000) then
        calories := 0;
      end if;
      hores := hores + 1;
      if (hores > 15) then
        hores := 0;
      end if;
      
      if (quants = 1) then
        AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'ES', '2', pases, calories, hores, RSP); 
      elsif (quants = 2) then
        AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'ES', '3', pases, calories, hores, RSP); 
      elsif (quants = 3) then
        AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'FR', '1', pases, calories, hores, RSP); 
      elsif (quants = 4) then
        AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'FR', '2', pases, calories, hores, RSP); 
      elsif (quants = 5) then
        AddPhysicalActivityData( TO_DATE( TO_CHAR(dia) || '-' || TO_CHAR(mes) || '-2014', 'DD-MM-YYYY'), 'DK', '3', pases, calories, hores, RSP); 
      end if;
      
    end loop;
  end loop;
end loop;

-- Aquest llistat es pot veure per pantalla fent:
ShowCountryActivityList(2014, 30, RSP);
if (RSP <> 'OK') then
  quants := 0;
  RSP := '';
  missatge := 'Resultat errni del llistat d''activitat fsica dels pasos.';
  goto final_error;
end if;

-- Llistat dels ciutadas utilitzats en aquest test.
--DBMS_OUTPUT.PUT_LINE('- Ciutadans que es fan servir en aquest test...');
--select "CODIPAIS", "IDCIUTADA", "DATANAIXEMENT", 
--  (EXTRACT(year from tests_date) - EXTRACT(year from "DATANAIXEMENT")) as "Edat" 
--  from ciutadans
--  order by "Edat";
--DBMS_OUTPUT.PUT_LINE('');

-- Verificar nmero de registres del llistat.
DBMS_OUTPUT.PUT_LINE('- Nmero de registres del llistat...');
select count(*) into quants 
  from TABLE(funcActFisicaMitjaPaisos(2014, 30)); 
if (quants <> 28) then
  RSP := '';
  missatge := 'Resultat errni del llistat d''activitat fsica dels pasos.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Nmero de registres del llistat edat = 30...');
select count(*) into quants 
  from TABLE(funcActFisicaMitjaPaisos(2014, 30))
  where "MITJAPASSES" > 0; 
if (quants <> 3) then
  RSP := '';
  missatge := 'Resultat errni del llistat d''activitat fsica dels pasos edat = 30.';
  goto final_error;
end if;
DBMS_OUTPUT.PUT_LINE('- Nmero de registres del llistat edat = 25...');
select count(*) into quants 
  from TABLE(funcActFisicaMitjaPaisos(2014, 25))
  where "MITJAPASSES" > 0; 
if (quants <> 2) then
  RSP := '';
  missatge := 'Resultat errni del llistat d''activitat fsica dels pasos edat = 25.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais DK edat = 30...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 30))
  where "CODIPAIS" = 'DK'; 
if (pases <> 1045) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas DK edat = 30.';
  goto final_error;
end if;
if (calories <> 0) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas DK edat = 30.';
  goto final_error;
end if;
if (hores <> 0) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas DK edat = 30.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais ES edat = 30...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 30))
  where "CODIPAIS" = 'ES'; 
if (pases <> 1005) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas ES edat = 30.';
  goto final_error;
end if;
if (calories <> 5353) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas ES edat = 30.';
  goto final_error;
end if;
if (hores <> 8) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas ES edat = 30.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais FR edat = 30...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 30))
  where "CODIPAIS" = 'FR'; 
if (pases <> 1030) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas FR edat = 30.';
  goto final_error;
end if;
if (calories <> 0) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas FR edat = 30.';
  goto final_error;
end if;
if (hores <> 0) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas FR edat = 30.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais DK edat = 25...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 25))
  where "CODIPAIS" = 'DK'; 
if (pases <> 1045) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas DK edat = 25.';
  goto final_error;
end if;
if (calories <> 5250) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas DK edat = 25.';
  goto final_error;
end if;
if (hores <> 8) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas DK edat = 25.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais ES edat = 25...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 25))
  where "CODIPAIS" = 'ES'; 
if (pases <> 0) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas ES edat = 25.';
  goto final_error;
end if;
if (calories <> 0) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas ES edat = 25.';
  goto final_error;
end if;
if (hores <> 0) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas ES edat = 25.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('- Nmero mitj de passes, calories i hores del pais FR edat = 25...');
select "MITJAPASSES", "MITJACALORIES", "MITJAHORESINACTIVITAT"
  into pases, calories, hores
  from TABLE(funcActFisicaMitjaPaisos(2014, 25))
  where "CODIPAIS" = 'FR'; 
if (pases <> 1030) then
  quants := pases;
  RSP := '';
  missatge := 'Resultat errni de la mitja de passes del pas FR edat = 25.';
  goto final_error;
end if;
if (calories <> 5250) then
  quants := calories;
  RSP := '';
  missatge := 'Resultat errni de la mitja de calories del pas FR edat = 25.';
  goto final_error;
end if;
if (hores <> 8) then
  quants := hores;
  RSP := '';
  missatge := 'Resultat errni de la mitja d''hores d''inactivitat del pas FR edat = 25.';
  goto final_error;
end if;

-- Verificar els registres del llistat.
DBMS_OUTPUT.PUT_LINE('- Nmero de pases, calories i hores del ciutad ES 1 al Gener...');

-- --------------------------------------------------------
-- Test especfics per a testejar la estadstica 9.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('TESTS D''ESTADSTICA DE CIUTAD QUE MS CALORIES HA GASTAT');
DBMS_OUTPUT.PUT_LINE( LPAD( '-', 80, '-' ));
DBMS_OUTPUT.PUT_LINE('');

DBMS_OUTPUT.PUT_LINE('- Estadstica 9 : ciutad que ms calories ha cremat en un any ...');

-- Esborrar les dades existents.
delete from rankingcalories;
delete from estadisticacalories;
delete from estadisticavalors;
delete from ussistema;
delete from estatsalut;
delete from activitatfisica;
delete from ciutadans;
delete from logprocediments;

CreateCitizen('ES', '1', 'Juan', 'Lpez', 'H', 'juanlopez@email.com', tests_date - 30 * 365, RSP);
CreateCitizen('ES', '2', 'Pio', 'Lpez', 'H', 'piolopez@email.com', tests_date - 25 * 365, RSP);
CreateCitizen('ES', '3', 'Maria', 'Gmis', 'D', 'mariagomis@email.com', tests_date - 30 * 365, RSP);
CreateCitizen('FR', '1', 'Fraoise', 'Gibernau', 'H', 'fracoisegibernau@email.com', tests_date - 25 * 365, RSP);
CreateCitizen('FR', '2', 'Alain', 'Martin', 'H', 'alainmartin@email.com', tests_date - 60 * 365, RSP);
CreateCitizen('DK', '3', 'Mery', 'Danks', 'D', 'merydanks@email.com', tests_date - 80 * 365, RSP);

DBMS_OUTPUT.PUT_LINE('  - Quan no hi ha cap registre ...');
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 0) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;  
if (RSP <> 'OK' or quants <> 0 or pais <> '*') then
  DBMS_OUTPUT.PUT_LINE('    ES: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - sense registres.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 0) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;  
if (RSP <> 'OK' or quants <> 0 or pais <> '*') then
  DBMS_OUTPUT.PUT_LINE('    UE: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - sense registres.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Quan noms hi ha un registre d''un ciutad ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 100, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 100) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;  
if (RSP <> 'OK' or quants <> 100 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - quan noms hi ha 1 registre.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 100) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;  
if (RSP <> 'OK' or quants <> 100 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    UE: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - quan noms hi ha 1 registre.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre del mateix ciutad ...');
AddPhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 100, 0, RSP); 
if (RSP <> 'OK') then 
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 200) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 200 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - Afegir un registre del mateix ciutad.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 200) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 200 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    UE: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - Afegir un registre del mateix ciutad.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre d''un altre ciutad amb menys calories ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', 0, 100, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 200) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 200 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - un registre d''un altre ciutad amb menys calories.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 200) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 200 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - un registre d''un altre ciutad amb menys calories.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre d''un altre ciutad amb ms calories ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '3', 0, 300, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - un registre d''un altre ciutad amb ms calories.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - un registre d''un altre ciutad amb ms calories.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre d''un altre ciutad amb ms calories encara ...');
AddPhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '2', 0, 500, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 600) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 600 or pais <> 'ES' or ciutada <> '2') then
  DBMS_OUTPUT.PUT_LINE('    ES: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - un registre d''un altre ciutad amb ms calories encara.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 600) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 600 or pais <> 'ES' or ciutada <> '2') then
  DBMS_OUTPUT.PUT_LINE('    UE: ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - un registre d''un altre ciutad amb ms calories encara.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Eliminar un registre del ciutad amb ms calories, sense provocar canvi a la estadstica ...');
RemovePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '2', RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 500) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 500 or pais <> 'ES' or ciutada <> '2') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - eliminar registre sense provocar canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 500) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 500 or pais <> 'ES' or ciutada <> '2') then
  DBMS_OUTPUT.PUT_LINE('    ' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - eliminar registre sense provocar canvi.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Eliminar un registre del ciutad amb ms calories, provocant canvi a la estadstica ...');
AddPhysicalActivityData( TO_DATE('03-12-2014', 'DD-MM-YYYY'), 'ES', '2', 0, 100, 0, RSP);
RemovePhysicalActivityData( TO_DATE('02-12-2014', 'DD-MM-YYYY'), 'ES', '2', RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - eliminar registre provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - eliminar registre provocant canvi.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Actualitzar un registre d''un altre ciutad, sense provocar canvi a la estadstica ...');
UpdatePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 150, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - Actualitzar registre sense provocar canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 300) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 300 or pais <> 'ES' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - Actualitzar registre sense provocar canvi.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Actualitzar un registre d''un altre ciutad, provocant canvi a la estadstica ...');
UpdatePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'ES', '1', 0, 600, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - Actualitzar registre provocant canvi a la estadstica.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - Actualitzar registre provocant canvi a la estadstica.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre d''un ciutad d''un altre pas, sense provocar canvi a la estadstica de la UE ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'FR', '1', 0, 400, 0, RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas ES.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - afegir ciutada d''altre pas sense provocar canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica del pas FR.
quants := funcObtenirValorEstadistica(9, 2014, 'FR', RSP);
if (RSP = 'OK' and quants = 400) then
  GetMostSpenderCitizen(2014, 'FR', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 400 or pais <> 'FR' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    FR:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 FR - afegir ciutada d''altre pas sense provocar canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - afegir ciutada d''altre pas sense provocar canvi.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Afegir un registre d''un ciutad d''un altre pas amb ms calories, provocant canvi a la estadstica de la UE ...');
AddPhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'DK', '3', 0, 1000, 0, RSP);
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas ES.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica del pas DK.
quants := funcObtenirValorEstadistica(9, 2014, 'DK', RSP);
if (RSP = 'OK' and quants = 1000) then
  GetMostSpenderCitizen(2014, 'DK', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 1000 or pais <> 'DK' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    DK:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 DK - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 1000) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 1000 or pais <> 'DK' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;

DBMS_OUTPUT.PUT_LINE('  - Modificar un registre d''un ciutad d''un altre pas amb ms calories, provocant canvi a la estadstica de la UE ...');
UpdatePhysicalActivityData( TO_DATE('01-12-2014', 'DD-MM-YYYY'), 'FR', '1', 0, 2000, 0, RSP); 
if (RSP <> 'OK') then
  goto final_error;
end if;  
-- Comprovar la estadstica del pas ES.
quants := funcObtenirValorEstadistica(9, 2014, 'ES', RSP);
if (RSP = 'OK' and quants = 700) then
  GetMostSpenderCitizen(2014, 'ES', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 700 or pais <> 'ES' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    ES:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 ES - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica del pas DK.
quants := funcObtenirValorEstadistica(9, 2014, 'DK', RSP);
if (RSP = 'OK' and quants = 1000) then
  GetMostSpenderCitizen(2014, 'DK', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 1000 or pais <> 'DK' or ciutada <> '3') then
  DBMS_OUTPUT.PUT_LINE('    DK:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 DK - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica del pas FR.
quants := funcObtenirValorEstadistica(9, 2014, 'FR', RSP);
if (RSP = 'OK' and quants = 2000) then
  GetMostSpenderCitizen(2014, 'FR', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 2000 or pais <> 'FR' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    FR:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 FR - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;
-- Comprovar la estadstica de la UE.
quants := funcObtenirValorEstadistica(9, 2014, 'UE', RSP);
if (RSP = 'OK' and quants = 2000) then
  GetMostSpenderCitizen(2014, 'UE', pais, ciutada, RSP);
end if;
if (RSP <> 'OK' or quants <> 2000 or pais <> 'FR' or ciutada <> '1') then
  DBMS_OUTPUT.PUT_LINE('    UE:' || quants || ' - ' || pais || ' - ' || ciutada);
  RSP := '';
  missatge := 'Resultat erroni de la estadstica 9 2014 UE - afegir ciutada d''altre pas provocant canvi.';
  goto final_error;
end if;

-- --------------------------------------------------------
-- Finalitzaci correcta de tots els tests.
-- --------------------------------------------------------

DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE( LPAD( '*', 80, '*' ));
DBMS_OUTPUT.PUT_LINE(' TOTS ELS TESTS S''HAN PASSAT CORRECTAMENT.');
DBMS_OUTPUT.PUT_LINE('');

goto final_tests;

-- --------------------------------------------------------
-- Finalitzaci amb errors.
-- --------------------------------------------------------

<<final_error>>
DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE('**** ATENCI: HA FALLAT EL TEST AMB EL RESULTAT: ');
DBMS_OUTPUT.PUT_LINE('     ' || RSP || ' ' || missatge || ' = ' || quants);
DBMS_OUTPUT.PUT_LINE('     Per obtenir ms informaci, sisplau revisa el log.');

-- --------------------------------------------------------
-- Finalitzaci del testing.
-- --------------------------------------------------------

<<final_tests>>
DBMS_OUTPUT.PUT_LINE('');
DBMS_OUTPUT.PUT_LINE(' FINAL DELS TESTS.');
DBMS_OUTPUT.PUT_LINE( LPAD( '*', 80, '*' ));

end;

-- ***************************************************************************
-- Final del script.
-- ***************************************************************************
